@@ -32,9 +32,10 @@ using namespace TelEngine;
 
 IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
-	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval)
+	u_int32_t format, u_int32_t capab, u_int32_t trunkSendInterval, bool authRequired)
     : Mutex(true),
     m_lastGetEvIndex(0),
+    m_authRequired(authRequired),
     m_maxFullFrameDataLen(maxFullFrameDataLen),
     m_startLocalCallNo(0),
     m_transListCount(0),
@@ -48,6 +49,8 @@ IAXEngine::IAXEngine(const char* iface, int port, u_int16_t transListCount, u_in
     m_trunkSendInterval(trunkSendInterval)
 {
     debugName("iaxengine");
+    Debug(this,DebugAll,"Automatically request authentication set to '%s'.",
+	(authRequired?"YES":"NO"));
     if ((port <= 0) || port > 65535)
 	port = 4569;
     if (transListCount < 4)
@@ -173,6 +176,13 @@ IAXTransaction* IAXEngine::addFrame(const SocketAddr& addr, const unsigned char*
     IAXFrame* frame = IAXFrame::parse(buf,len,this,&addr);
     if (!frame)
 	return 0;
+    if (frame->fullFrame() && debugAt(DebugAll)) {
+	String s;
+	SocketAddr local;
+	m_socket.getSockName(local);
+	frame->fullFrame()->toString(s,local,addr,true);
+	Debug(this,DebugInfo,"Received frame.%s",s.c_str());
+    }
     IAXTransaction* tr = addFrame(addr,frame);
     if (!tr)
 	frame->deref();
@@ -215,8 +225,15 @@ void IAXEngine::readSocket(SocketAddr& addr)
     }
 }
 
-bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr)
+bool IAXEngine::writeSocket(const void* buf, int len, const SocketAddr& addr, IAXFullFrame* frame)
 {
+    if (frame && debugAt(DebugAll)) {
+	String s;
+	SocketAddr local;
+	m_socket.getSockName(local);
+	frame->toString(s,local,addr,false);
+	Debug(this,DebugInfo,"Sending frame.%s",s.c_str());
+    }
     len = m_socket.sendTo(buf,len,addr);
     if (len == Socket::socketError()) {
 	if (!m_socket.canRetry())
@@ -277,8 +294,13 @@ u_int32_t IAXEngine::transactionCount()
 
 void IAXEngine::keepAlive(SocketAddr& addr)
 {
+#if 0
     unsigned char buf[12] = {0x80,0,0,0,0,0,0,0,0,0,IAXFrame::IAX,IAXControl::Inval};
     writeSocket(buf,sizeof(buf),addr);
+#endif
+    IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Inval,0,0,0,0,0);
+    writeSocket(f->data().data(),f->data().length(),addr,f);
+    f->deref();
 }
 
 bool IAXEngine::processTrunkFrames(u_int32_t time)
@@ -400,9 +422,23 @@ bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans)
 	return false;
     u_int32_t format = trans->format();
     u_int32_t capability = m_capability & trans->capability();
+#ifdef XDEBUG
+    if (!trans->outgoing()) {
+	String rec;
+	IAXFormat::formatList(rec,m_capability);
+	Debug(this,DebugAll,"acceptFormatAndCapability. Local: Format: %u(%s). Capabilities: '%s'.",
+	    m_format,IAXFormat::audioText(m_format),rec.c_str());
+	IAXFormat::formatList(rec,trans->capability());
+	Debug(this,DebugAll,"acceptFormatAndCapability. Received: Format: %u(%s). Capabilities: '%s'.",
+	    format,IAXFormat::audioText(format),rec.c_str());
+    }
+#endif //XDEBUG
     // Valid capability ?
-    if (!capability)
+    if (!capability) {
+	if (!trans->outgoing())
+	    XDebug(this,DebugAll,"acceptFormatAndCapability. No capabilities received.");
 	return false;
+    }
     for (;;) {
 	// Received format is valid ?
 	if (0 != (format & capability) && IAXFormat::audioText(format))
@@ -421,13 +457,15 @@ bool IAXEngine::acceptFormatAndCapability(IAXTransaction* trans)
 	    format = IAXFormat::audioData[i].value;
 	    break;
 	}
+	XDebug(this,DebugAll,"acceptFormatAndCapability. Unable to choose a format.");
 	return false;
     }
+    XDebug(this,DebugAll,"acceptFormatAndCapability. Format %u: '%s'.",
+	format,IAXFormat::audioText(format));
     trans->m_format = format;
     trans->m_capability = capability;
-    if (trans->outgoing())
-	trans->m_formatIn = format;
-    else
+    trans->m_formatIn = format;
+    if (!trans->outgoing())
 	trans->m_formatOut = format;
     return true;
 }
