@@ -30,6 +30,7 @@ using namespace TelEngine;
 String IAXTransaction::s_iax_modNoAuthMethod("Unsupported or missing authentication method or missing challenge");
 String IAXTransaction::s_iax_modNoMediaFormat("Unsupported or missing media format or capability");
 String IAXTransaction::s_iax_modInvalidAuth("Invalid authentication request, response or challenge");
+String IAXTransaction::s_iax_modNoUsername("Username is missing");
 
 unsigned char IAXTransaction::m_maxInFrames = 100;
 
@@ -139,9 +140,15 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
     m_capability(0),
     m_trunkFrame(0)
 {
-    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing [%p]",
+    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing. [%p]",
 	localCallNo(),remoteCallNo(),this);
     // Init data members
+    if (!m_addr.port()) {
+	XDebug(m_engine,DebugAll,
+	    "IAXTransaction::IAXTransaction(%u,%u). No remote port. Set to default. [%p]",
+	    localCallNo(),remoteCallNo(),this);
+	m_addr.port(4569);
+    }
     m_retransCount = engine->retransCount();
     m_retransInterval = engine->retransInterval();
     m_timeToNextPing = m_timeStamp + m_pingInterval;
@@ -171,16 +178,16 @@ IAXTransaction::IAXTransaction(IAXEngine* engine, Type type, u_int16_t lcallno,
 	    frametype = IAXControl::Poke;
 	    break;
 	default:
-	    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing [%p]. Unsupported type: %u",
-		localCallNo(),remoteCallNo(),this,m_type);
+	    XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u) outgoing. Unsupported type: %u. [%p]",
+		localCallNo(),remoteCallNo(),m_type,this);
 	    m_type = Incorrect;
 	    return;
     }
     DataBlock d;
     ieList.toBuffer(d);
     if (d.length() > (unsigned int)m_engine->maxFullFrameDataLen()) {
-	XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u)[%p]. Buffer too long (%u > %u)",
-		localCallNo(),remoteCallNo(),this,d.length(),(unsigned int)m_engine->maxFullFrameDataLen());
+	XDebug(m_engine,DebugAll,"IAXTransaction::IAXTransaction(%u,%u). Buffer too long (%u > %u). [%p]",
+		localCallNo(),remoteCallNo(),d.length(),(unsigned int)m_engine->maxFullFrameDataLen(),this);
 	d.clear();
     }
     postFrame(IAXFrame::IAX,frametype,(void*)(d.data()),d.length());
@@ -196,6 +203,8 @@ IAXTransaction::~IAXTransaction()
 	m_trunkFrame->deref();
     if (state() != Terminating && state() != Terminated)
 	sendReject("Server shutdown");
+    XDebug(m_engine,DebugAll,"IAXTransaction::~IAXTransaction(%u,%u). [%p]",
+	localCallNo(),remoteCallNo(),this);
 }
 
 IAXTransaction* IAXTransaction::factoryIn(IAXEngine* engine, IAXFullFrame* frame, u_int16_t lcallno, 
@@ -253,8 +262,13 @@ IAXTransaction* IAXTransaction::processFrame(IAXFrame* frame)
 	return 0;
     incrementSeqNo(frame->fullFrame(),true);
     // Voice full frame: process voice data & format
-    if (frame->type() == IAXFrame::Voice && type() == New)
-	return processVoiceFrame(frame->fullFrame());
+    if (frame->type() == IAXFrame::Voice && type() == New) {
+	if (!processVoiceFrame(frame->fullFrame()))
+	    return 0;
+	// Frame accepted: process voice data
+	lock.drop();
+	return processMedia(frame->data(),frame->timeStamp(),true);
+    }
     // Process incoming Ping
     if (frame->type() == IAXFrame::IAX && frame->fullFrame()->subclass() == IAXControl::Ping) {
 	DDebug(m_engine,DebugAll,"Transaction(%u,%u) received Ping iseq=%u oseq=%u stamp=%u [%p]",
@@ -340,6 +354,7 @@ IAXEvent* IAXTransaction::getEvent(u_int64_t time)
     GenObject* obj;
     bool delFrame;
 
+    Lock lock(this);
     if (state() == Terminated)
 	return 0;
     // Send ack for received frames
@@ -347,7 +362,6 @@ IAXEvent* IAXTransaction::getEvent(u_int64_t time)
     // Do we have a generated event ?
     if (m_currentEvent)
 	return 0;
-    Lock lock(this);
     // Waiting on remote cleanup ?
     if (state() == Terminating && !m_localReqEnd)
 	return getEventTerminating(time);
@@ -460,15 +474,14 @@ bool IAXTransaction::sendAccept()
 	((type() == RegReq || type() == RegRel) && state() == NewRemoteInvite_RepRecv)))
 	return false;
     if (type() == New) {
-	unsigned char d[6] = {IAXInfoElement::FORMAT,4,m_format >> 24,m_format >> 16,m_format >> 8,m_format};
+	unsigned char d[12] = {IAXInfoElement::FORMAT,4,m_format >> 24,m_format >> 16,m_format >> 8,m_format,
+		IAXInfoElement::CAPABILITY,4,m_capability >> 24,m_capability >> 16,m_capability >> 8,m_capability};
 	postFrame(IAXFrame::IAX,IAXControl::Accept,d,sizeof(d),0,true);
 	changeState(Connected);
     }
     else {
 	IAXIEList ieList;
 	ieList.appendString(IAXInfoElement::USERNAME,m_username);
-	ieList.appendString(IAXInfoElement::CALLING_NUMBER,m_callingNo);
-	ieList.appendString(IAXInfoElement::CALLING_NAME,m_callingName);
 	ieList.appendNumeric(IAXInfoElement::REFRESH,m_expire,2);
 	ieList.appendIE(IAXInfoElementBinary::packIP(remoteAddr()));
 	DataBlock data;
@@ -551,12 +564,11 @@ bool IAXTransaction::sendReject(const char* cause, u_int8_t code)
     return true;
 }
 
-bool IAXTransaction::sendAuth(const String& pwd)
+bool IAXTransaction::sendAuth()
 {
     Lock lock(this);
     if (!((type() == New || type() == RegReq || type() == RegRel) && state() == NewRemoteInvite))
 	return false;
-    m_password = pwd;
     switch (m_authmethod) {
 	case IAXAuthMethod::MD5:
 	    m_challenge = (int)random();
@@ -586,13 +598,13 @@ bool IAXTransaction::sendAuth(const String& pwd)
     return true;
 }
 
-bool IAXTransaction::sendAuthReply()
+bool IAXTransaction::sendAuthReply(const String& response)
 {
     Lock lock(this);
     if (state() != NewLocalInvite_AuthRecv)
 	return false;
+    m_authdata = response;
     IAXIEList ieList;
-    String authdata;
     IAXControl::Type subclass;
     switch (type()) {
 	case New:
@@ -611,8 +623,7 @@ bool IAXTransaction::sendAuthReply()
     }
     if (m_authmethod != IAXAuthMethod::MD5)
 	return false;
-    m_engine->getMD5FromChallenge(authdata,m_challenge,m_password);
-    ieList.appendString(IAXInfoElement::MD5_RESULT,authdata);
+    ieList.appendString(IAXInfoElement::MD5_RESULT,response);
     DataBlock data;
     ieList.toBuffer(data);
     postFrame(IAXFrame::IAX,subclass,data.data(),data.length(),0,false);
@@ -668,26 +679,22 @@ bool IAXTransaction::enableTrunking(IAXMetaTrunkFrame* trunkFrame)
 
 void IAXTransaction::print()
 {
-    Debug(m_engine,DebugInfo,"Transaction - START PRINT [%p]",this);
-    Output("Local call no:    %u\nRemote call no:   %u\nType:             %u\nState:            %u\nTimestamp:        " FMT64U,
-	localCallNo(),remoteCallNo(),type(),state(),(u_int64_t)timeStamp());
-    Output("Queues:\nOutgoing:         %u",m_outFrames.count());
+    static SocketAddr addr;
     ObjList* l;
-    int i;
-    for(i = 0, l = m_outFrames.skipNull(); l; l = l->next(), i++) {
+    String frames;
+    frames << "\r\nOutgoing frames: " << m_outFrames.count();
+    for(l = m_outFrames.skipNull(); l; l = l->skipNext()) {
 	IAXFrameOut* frame = static_cast<IAXFrameOut*>(l->get());
-	if (frame)
-	    Output("     %5u  Type: %-3u Subclass: %-3u Out: %-5u In: %-5u Timestamp: %-5u Ack: %u AckOnly: %u",
-		i+1,frame->type(),frame->subclass(),frame->oSeqNo(),frame->iSeqNo(), frame->timeStamp(),frame->ack(),frame->ackOnly());
+	frame->toString(frames,addr,remoteAddr(),false);
     }
-    Output("Incoming:         %u",m_inFrames.count());
-    for(i = 0, l = m_inFrames.skipNull(); l; l = l->next(), i++) {
-	IAXFullFrame* frame = static_cast<IAXFrameOut*>(l->get());
-	if (frame)
-	    Output("     %5u  Type: %-3u Subclass: %-3u Out: %-5u In: %-5u Timestamp: %-5u",
-		i+1,frame->type(),frame->subclass(),frame->oSeqNo(),frame->iSeqNo(), frame->timeStamp());
+    frames << "\r\nIncoming frames: " << m_inFrames.count();
+    for(l = m_inFrames.skipNull(); l; l = l->skipNext()) {
+	IAXFrameOut* frame = static_cast<IAXFrameOut*>(l->get());
+	frame->toString(frames,addr,remoteAddr(),true);
     }
-    Debug(m_engine,DebugInfo,"Transaction - END PRINT [%p]",this);
+    Debug(m_engine,DebugInfo,"Transaction(%u,%u). Remote address: %s:%u. Type: %u. State: %u. Timestamp: " FMT64U " [%p]%s",
+	localCallNo(),remoteCallNo(),remoteAddr().host().c_str(),remoteAddr().port(),
+	type(),state(),(u_int64_t)timeStamp(),this,frames.c_str());
 }
 
 void IAXTransaction::init(IAXIEList& ieList)
@@ -695,7 +702,6 @@ void IAXTransaction::init(IAXIEList& ieList)
     switch (type()) {
 	case New:
 	    ieList.getString(IAXInfoElement::USERNAME,m_username);
-	    ieList.getString(IAXInfoElement::PASSWORD,m_password);
 	    ieList.getString(IAXInfoElement::CALLING_NUMBER,m_callingNo);
 	    ieList.getString(IAXInfoElement::CALLING_NAME,m_callingName);
 	    ieList.getString(IAXInfoElement::CALLED_NUMBER,m_calledNo);
@@ -710,7 +716,6 @@ void IAXTransaction::init(IAXIEList& ieList)
 	case RegReq:
 	case RegRel:
 	    ieList.getString(IAXInfoElement::USERNAME,m_username);
-	    ieList.getString(IAXInfoElement::PASSWORD,m_password);
 	    ieList.getNumeric(IAXInfoElement::REFRESH,m_expire);
 	    break;
 	case Poke:
@@ -832,7 +837,7 @@ bool IAXTransaction::sendFrame(IAXFrameOut* frame, bool vnak)
 {
     if (!frame)
 	return false;
-    bool b = m_engine->writeSocket(frame->data().data(),frame->data().length(),remoteAddr());
+    bool b = m_engine->writeSocket(frame->data().data(),frame->data().length(),remoteAddr(),frame);
     // Don't modify timeout if transmitted as a response to a VNAK
     if (!vnak) {
 	if (frame->retrans())     // Retransmission
@@ -949,7 +954,8 @@ IAXEvent* IAXTransaction::getEventResponse_New(IAXFrameOut* frame, bool& delFram
 	    if (!(frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::AuthReq))
 		break;
 	    // Frame is AUTHREQ: AUTHREP, REJECT, HANGUP ?
-	    if (0 != (ev = createResponse(frame,IAXFrame::IAX,IAXControl::New,IAXEvent::AuthRep,false,NewRemoteInvite_RepRecv)))
+	    if (0 != (ev = createResponse(frame,IAXFrame::IAX,IAXControl::AuthRep,
+		IAXEvent::AuthRep,false,NewRemoteInvite_RepRecv)))
 		return processAuthRep(ev);
 	    if (0 != (ev = createResponse(frame,IAXFrame::IAX,IAXControl::Reject,IAXEvent::Reject,false,Terminating)))
 		return ev;
@@ -972,11 +978,8 @@ IAXEvent* IAXTransaction::processAuthReq(IAXEvent* event)
     u_int32_t authmethod;
     bool bAuthMethod = event->getList().getNumeric(IAXInfoElement::AUTHMETHODS,authmethod) && (authmethod & m_authmethod);
     bool bChallenge = event->getList().getString(IAXInfoElement::CHALLENGE,m_challenge);
-    if (bAuthMethod && bChallenge) {
-	Debug(m_engine,DebugAll,"Transaction(%u,%u). Internal authentication reply",localCallNo(),remoteCallNo());
-	sendAuthReply();
+    if (bAuthMethod && bChallenge)
 	return event;
-    }
     delete event;
     return internalReject(s_iax_modNoAuthMethod);
 }
@@ -1003,12 +1006,6 @@ IAXEvent* IAXTransaction::processAuthRep(IAXEvent* event)
 	return event;
     Debug(m_engine,DebugAll,"Transaction(%u,%u). Auth Reply received",localCallNo(),remoteCallNo());
     event->getList().getString(IAXInfoElement::MD5_RESULT,m_authdata);
-    if (type() == RegReq || type() == RegRel) {
-        delete event;
-	if (!IAXEngine::isMD5ChallengeCorrect(m_authdata,m_challenge,m_password))
-	    return internalReject(s_iax_modInvalidAuth);
-	return internalAccept();
-    }
     return event;
 }
 
@@ -1080,12 +1077,16 @@ IAXEvent* IAXTransaction::getEventStartTrans(IAXFullFrame* frame, bool& delFrame
 		break;
 	    ev = createEvent(IAXEvent::New,false,frame,NewRemoteInvite);
 	    if (ev) {
+		// Check version
 		if (!ev->getList().validVersion()) {
 		    delete ev;
 		    sendReject("Unsupported or missing protocol version");
 		    return 0;
 		}
 		init(ev->getList());
+		// Check username
+		if (!m_username)
+		    return internalReject(s_iax_modNoUsername);
 	    }
 	    return ev;
 	case RegReq:
@@ -1095,6 +1096,9 @@ IAXEvent* IAXTransaction::getEventStartTrans(IAXFullFrame* frame, bool& delFrame
 		break;
 	    ev = createEvent(IAXEvent::New,false,frame,NewRemoteInvite);
 	    init(ev->getList());
+	    // Check username
+	    if (!m_username)
+		return internalReject(s_iax_modNoUsername);
 	    return ev;
 	case Poke:
 	    if (!(frame->type() == IAXFrame::IAX && frame->subclass() == IAXControl::Poke))
@@ -1189,11 +1193,17 @@ IAXFullFrame* IAXTransaction::findInFrame(IAXFrame::Type type, u_int32_t subclas
 bool IAXTransaction::findInFrameTimestamp(const IAXFullFrame* frameOut, IAXFrame::Type type, u_int32_t subclass)
 {
     IAXFullFrame* frame = 0;
-    for (ObjList* l = m_inFrames.skipNull(); l; l = l->next()) {
+    // Loose timestamp check for Ping/Pong
+    // Received timestamp can be greater then the sent one
+    bool looseTimestamp = type == IAXFrame::IAX && subclass == IAXControl::Pong;
+    for (ObjList* l = m_inFrames.skipNull(); l; l = l->skipNext()) {
 	frame = static_cast<IAXFullFrame*>(l->get());
-	if (frame && frame->type() == type && frame->subclass() == subclass && frame->timeStamp() == frameOut->timeStamp())
-	    break;
-	frame = 0;
+	if (frame->type() == type && frame->subclass() == subclass) {
+	    bool match = looseTimestamp ? frame->timeStamp() >= frameOut->timeStamp() :
+		frame->timeStamp() == frameOut->timeStamp();
+	    if (match)
+		break;
+	}
     }
     if (frame) {
 	m_inFrames.remove(frame,true);
@@ -1249,30 +1259,28 @@ void IAXTransaction::sendAck(const IAXFullFrame* frame)
 {
     if (!frame)
 	return;
-    unsigned char buf[12] = {0x80 | localCallNo() >> 8,localCallNo(),remoteCallNo() >> 8,remoteCallNo(),
-			     frame->timeStamp() >> 24,frame->timeStamp() >> 16,frame->timeStamp() >> 8,frame->timeStamp(),
-			     frame->iSeqNo(),m_iSeqNo,IAXFrame::IAX,IAXControl::Ack};
+    IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Ack,localCallNo(),
+	remoteCallNo(),frame->iSeqNo(),m_iSeqNo,frame->timeStamp());
     DDebug(m_engine,DebugInfo,"Transaction(%u,%u). Send ACK for Frame(%u,%u) oseq: %u iseq: %u",
 	    localCallNo(),remoteCallNo(),frame->type(),frame->subclass(),frame->oSeqNo(),frame->iSeqNo());
-    m_engine->writeSocket(buf,sizeof(buf),remoteAddr());
+    m_engine->writeSocket(f->data().data(),f->data().length(),remoteAddr(),f);
+    f->deref();
 }
 
 void IAXTransaction::sendInval()
 {
-    u_int32_t ts = (u_int32_t)timeStamp();
-    unsigned char buf[12] = {0x80 | localCallNo() >> 8,localCallNo(),remoteCallNo() >> 8,remoteCallNo(),
-			     ts >> 24,ts >> 16,ts >> 8,ts,
-			     m_oSeqNo++,m_iSeqNo,IAXFrame::IAX,IAXControl::Inval};
-    m_engine->writeSocket(buf,sizeof(buf),remoteAddr());
+    IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::Inval,localCallNo(),
+	remoteCallNo(),m_oSeqNo++,m_iSeqNo,(u_int32_t)timeStamp());
+    m_engine->writeSocket(f->data().data(),f->data().length(),remoteAddr(),f);
+    f->deref();
 }
 
 void IAXTransaction::sendVNAK()
 {
-    u_int32_t ts = (u_int32_t)timeStamp();
-    unsigned char buf[12] = {0x80 | localCallNo() >> 8,localCallNo(),remoteCallNo() >> 8,remoteCallNo(),
-			     ts >> 24,ts >> 16,ts >> 8,ts,
-			     m_oSeqNo,m_iSeqNo,IAXFrame::IAX,IAXControl::VNAK};
-    m_engine->writeSocket(buf,sizeof(buf),remoteAddr());
+    IAXFullFrame* f = new IAXFullFrame(IAXFrame::IAX,IAXControl::VNAK,localCallNo(),
+	remoteCallNo(),m_oSeqNo,m_iSeqNo,(u_int32_t)timeStamp());
+    m_engine->writeSocket(f->data().data(),f->data().length(),remoteAddr(),f);
+    f->deref();
 }
 
 void IAXTransaction::sendUnsupport(u_int32_t subclass)
@@ -1452,9 +1460,7 @@ IAXTransaction* IAXTransaction::processVoiceFrame(const IAXFullFrame* frame)
 	    return 0;
 	}
     }
-    // Process voice data
-    processMedia(((IAXFullFrame*)frame)->data(),frame->timeStamp(),true);
-    return 0;
+    return this;
 }
 
 IAXTransaction* IAXTransaction::retransmitOnVNAK(u_int16_t seqNo)
@@ -1489,4 +1495,14 @@ IAXEvent* IAXTransaction::internalReject(String& reason)
     return event;
 }
 
+void IAXTransaction::eventTerminated(IAXEvent* event)
+{
+    Lock lock(this);
+    if (event && event == m_currentEvent) {
+	XDebug(m_engine,DebugAll,"Transaction(%u,%u). Event (%p) terminated. [%p]",
+	    localCallNo(),remoteCallNo(),event,this);
+	m_currentEvent = 0;
+    }
+}
+
 /* vi: set ts=8 sw=4 sts=4 noet: */
