@@ -257,7 +257,7 @@ SIPMessage::SIPMessage(const SIPMessage& original)
 }
 
 SIPMessage::SIPMessage(const char* _method, const char* _uri, const char* _version)
-    : version(_version), method(_method), uri(_uri),
+    : version(_version), method(_method), uri(_uri), code(0),
       body(0), m_ep(0), m_valid(true),
       m_answer(false), m_outgoing(true), m_ack(false), m_cseq(-1)
 {
@@ -266,7 +266,8 @@ SIPMessage::SIPMessage(const char* _method, const char* _uri, const char* _versi
 }
 
 SIPMessage::SIPMessage(SIPParty* ep, const char* buf, int len)
-    : body(0), m_ep(ep), m_valid(false), m_answer(false), m_outgoing(false), m_ack(false), m_cseq(-1)
+    : code(0), body(0), m_ep(ep), m_valid(false),
+      m_answer(false), m_outgoing(false), m_ack(false), m_cseq(-1)
 {
     DDebug(DebugInfo,"SIPMessage::SIPMessage(%p,%d) [%p]\n------\n%s------",
 	buf,len,this,buf);
@@ -309,7 +310,7 @@ SIPMessage::SIPMessage(const SIPMessage* message, int _code, const char* _reason
 }
 
 SIPMessage::SIPMessage(const SIPMessage* original, const SIPMessage* answer)
-    : method("ACK"),
+    : method("ACK"), code(0),
       body(0), m_ep(0), m_valid(false),
       m_answer(false), m_outgoing(true), m_ack(true), m_cseq(-1)
 {
@@ -341,8 +342,12 @@ SIPMessage::SIPMessage(const SIPMessage* original, const SIPMessage* answer)
 	    if (uri.matches(r))
 		uri = uri.matchString(1);
 	}
-	if (!original->getHeader("Route"))
-	    copyAllHeaders(answer,"Record-Route","Route");
+	// new transaction - get/apply routeset unless INVITE already knew it
+	if (!original->getHeader("Route")) {
+	    ObjList* routeset = answer->getRoutes();
+	    addRoutes(routeset);
+	    delete routeset;
+	}
     }
     copyAllHeaders(original,"Route");
     copyHeader(original,"From");
@@ -808,6 +813,10 @@ SIPAuthLine* SIPMessage::buildAuth(const String& username, const String& passwor
 	    auth->setParam("uri",quote(msguri));
 	    auth->setParam("response",quote(response));
 	    auth->setParam("algorithm","MD5");
+	    // copy opaque data as-is, only if present
+	    const NamedString* opaque = t->getParam("opaque");
+	    if (opaque)
+		auth->setParam(opaque->name(),*opaque);
 	    return auth;
 	}
     }
@@ -854,11 +863,11 @@ ObjList* SIPMessage::getRoutes() const
 		if (!list)
 		    list = new ObjList;
 		if (isAnswer())
-		    // route set learned from an answer, preserve order
-		    list->append(line);
-		else
-		    // route set learned from a request, reverse order
+		    // route set learned from an answer, reverse order
 		    list->insert(line);
+		else
+		    // route set learned from a request, preserve order
+		    list->append(line);
 	    }
 	}
     }
