@@ -216,6 +216,21 @@ DataEndpoint* CallEndpoint::setEndpoint(const char* type)
     return dat;
 }
 
+void CallEndpoint::setEndpoint(DataEndpoint* endPoint)
+{
+    if (!(endPoint && endPoint->ref()))
+	return;
+    if (m_data.find(endPoint)) {
+	endPoint->deref();
+	return;
+    }
+    clearEndpoint(endPoint->toString());
+    endPoint->disconnect();
+    m_data.append(endPoint);
+    if (m_peer)
+	endPoint->connect(m_peer->getEndpoint(endPoint->toString()));
+}
+
 void CallEndpoint::clearEndpoint(const char* type)
 {
     if (null(type)) {
@@ -265,14 +280,18 @@ DataConsumer* CallEndpoint::getConsumer(const char* type) const
 
 Channel::Channel(Driver* driver, const char* id, bool outgoing)
     : CallEndpoint(id),
-      m_driver(driver), m_outgoing(outgoing), m_timeout(0), m_maxcall(0), m_answered(false)
+      m_driver(driver), m_outgoing(outgoing),
+      m_timeout(0), m_maxcall(0),
+      m_sequence(0), m_answered(false)
 {
     init();
 }
 
 Channel::Channel(Driver& driver, const char* id, bool outgoing)
     : CallEndpoint(id),
-      m_driver(&driver), m_outgoing(outgoing), m_timeout(0), m_maxcall(0), m_answered(false)
+      m_driver(&driver), m_outgoing(outgoing),
+      m_timeout(0), m_maxcall(0),
+      m_sequence(0), m_answered(false)
 {
     init();
 }
@@ -409,6 +428,11 @@ void Channel::complete(Message& msg, bool minimal) const
     msg.setParam("id",id());
     if (m_driver)
 	msg.setParam("module",m_driver->name());
+    if ((msg == "chan.dtmf") && !msg.getParam("sequence")) {
+	// need to add sequence number used to detect reorders
+	Lock lock(mutex());
+	msg.addParam("sequence",String(m_sequence++));
+    }
 
     if (minimal)
 	return;
@@ -435,6 +459,29 @@ Message* Channel::message(const char* name, bool minimal, bool data)
     return msg;
 }
 
+Message* Channel::message(const char* name, const NamedList* original, const char* params, bool minimal, bool data)
+{
+    Message* msg = message(name,minimal,data);
+    if (original) {
+	if (!params)
+	    params = original->getValue("copyparams");
+	if (!null(params)) {
+	    String tmp(params);
+	    ObjList* lst = tmp.split(',',false);
+	    for (ObjList* l = lst; l; l = l->next()) {
+		String* s = static_cast<String*>(l->get());
+		if (!s)
+		    continue;
+		s->trimBlanks();
+		if (*s)
+		    msg->copyParam(*original,*s);
+	    }
+	    delete lst;
+	}
+    }
+    return msg;
+}
+
 bool Channel::startRouter(Message* msg)
 {
     if (!msg)
@@ -508,6 +555,26 @@ bool Channel::msgUpdate(Message& msg)
     return false;
 }
 
+bool Channel::msgMasquerade(Message& msg)
+{
+    if (m_billid.null())
+	m_billid = msg.getValue("billid");
+    if (msg == "call.answered") {
+	Debug(this,DebugInfo,"Masquerading answer operation [%p]",this);
+	m_maxcall = 0;
+	status("answered");
+    }
+    else if (msg == "call.progress") {
+	Debug(this,DebugInfo,"Masquerading progress operation [%p]",this);
+	status("progressing");
+    }
+    else if (msg == "call.ringing") {
+	Debug(this,DebugInfo,"Masquerading ringing operation [%p]",this);
+	status("ringing");
+    }
+    return false;
+}
+
 void Channel::msgStatus(Message& msg)
 {
     String par;
@@ -516,7 +583,7 @@ void Channel::msgStatus(Message& msg)
     statusParams(par);
     lock.drop();
     msg.retValue().clear();
-    msg.retValue() << "name=" << id() << ",type=channel;" << par << "\n";
+    msg.retValue() << "name=" << id() << ",type=channel;" << par << "\r\n";
 }
 
 void Channel::statusParams(String& str)
@@ -638,7 +705,7 @@ bool Channel::dtmfInband(const char* tone)
 bool Channel::toneDetect(const char* sniffer)
 {
     if (null(sniffer))
-	return false;
+	sniffer = "tone/*";
     Message m("chan.attach");
     complete(m,true);
     m.userData(this);
@@ -663,7 +730,7 @@ bool Channel::setDebug(Message& msg)
 	debugEnabled(str.toBoolean(debugEnabled()));
     msg.retValue() << "Channel " << id()
 	<< " debug " << (debugEnabled() ? "on" : "off")
-	<< " level " << debugLevel() << (debugChained() ? " chained" : "") << "\n";
+	<< " level " << debugLevel() << (debugChained() ? " chained" : "") << "\r\n";
     return true;
 }
 
@@ -763,6 +830,7 @@ void Module::setup()
     installRelay(Timer,90);
     installRelay(Status,110);
     installRelay(Level,120);
+    installRelay(Command,120);
 }
 
 void Module::changed()
@@ -787,14 +855,46 @@ bool Module::msgRoute(Message& msg)
     return false;
 }
 
+bool Module::msgCommand(Message& msg)
+{
+    const NamedString* line = msg.getParam("line");
+    if (line)
+	return commandExecute(msg.retValue(),*line);
+    if (msg.getParam("partline") || msg.getParam("partword"))
+	return commandComplete(msg,msg.getValue("partline"),msg.getValue("partword"));
+    return false;
+}
+
+bool Module::commandExecute(String& retVal, const String& line)
+{
+    return false;
+}
+
+bool Module::commandComplete(Message& msg, const String& partLine, const String& partWord)
+{
+    if ((partLine == "debug") || (partLine == "status")) {
+	if (partWord.null() || name().startsWith(partWord)) {
+	    msg.retValue().append(name(),"\t");
+	    return false;
+	}
+    }
+    return false;
+}
+
 void Module::msgStatus(Message& msg)
 {
-    String mod, par;
+    String mod, par, det;
+    bool details = msg.getBoolValue("details",true);
     lock();
     statusModule(mod);
     statusParams(par);
+    if (details)
+	statusDetail(det);
     unlock();
-    msg.retValue() << mod << ";" << par << "\n";
+    msg.retValue() << mod << ";" << par;
+    if (det)
+	msg.retValue() << ";" << det;
+    msg.retValue() << "\r\n";
 }
 
 void Module::statusModule(String& str)
@@ -808,6 +908,10 @@ void Module::statusParams(String& str)
 {
 }
 
+void Module::statusDetail(String& str)
+{
+}
+
 void Module::genUpdate(Message& msg)
 {
 }
@@ -840,6 +944,8 @@ bool Module::received(Message &msg, int id)
     }
     else if (id == Level)
 	return setDebug(msg,dest);
+    else if (id == Command)
+	return msgCommand(msg);
 
     return false;
 }
@@ -871,7 +977,7 @@ bool Module::setDebug(Message& msg, const String& target)
 	<< " level " << debugLevel();
     if (m_filter)
 	msg.retValue() << " filter: " << m_filter;
-    msg.retValue() << "\n";
+    msg.retValue() << "\r\n";
     return true;
 }
 
@@ -957,10 +1063,6 @@ bool Driver::received(Message &msg, int id)
 		    if (!c)
 			break;
 		    c->checkTimers(msg,t);
-		    if (c->timeout() && (c->timeout() < t))
-			c->msgDrop(msg,"timeout");
-		    else if (c->maxcall() && (c->maxcall() < t))
-			c->msgDrop(msg,"noanswer");
 		    lock();
 		}
 	    }
@@ -971,6 +1073,7 @@ bool Driver::received(Message &msg, int id)
 		break;
 	case Level:
 	case Route:
+	case Command:
 	    return Module::received(msg,id);
 	case Halt:
 	    dropAll(msg);
@@ -985,6 +1088,9 @@ bool Driver::received(Message &msg, int id)
 	    break;
 	default:
 	    dest = msg.getValue("targetid");
+	    // if this channel is not the target try to match it as peer
+	    if (!dest.startsWith(m_prefix))
+		dest = msg.getValue("peerid");
 	    break;
     }
     XDebug(DebugAll,"id=%d prefix='%s' dest='%s'",id,m_prefix.c_str(),dest.c_str());
@@ -1041,7 +1147,7 @@ bool Driver::received(Message &msg, int id)
 	    msg.clearParam("message");
 	    msg.userData(chan);
 	    chan->complete(msg);
-	    return false;
+	    return chan->msgMasquerade(msg);
 	case Locate:
 	    msg.userData(chan);
 	    return true;
@@ -1099,13 +1205,18 @@ void Driver::genUpdate(Message& msg)
 
 void Driver::msgStatus(Message& msg)
 {
-    String mod, par, c;
+    String mod, par, det;
+    bool details = msg.getBoolValue("details",true);
     lock();
     statusModule(mod);
     statusParams(par);
-    statusChannels(c);
+    if (details)
+	statusDetail(det);
     unlock();
-    msg.retValue() << mod << ";" << par << ";" << c << "\n";
+    msg.retValue() << mod << ";" << par;
+    if (details)
+	msg.retValue() << ";" << det;
+    msg.retValue() << "\r\n";
 }
 
 void Driver::statusModule(String& str)
@@ -1123,7 +1234,7 @@ void Driver::statusParams(String& str)
     str << ",chans=" << m_chans.count();
 }
 
-void Driver::statusChannels(String& str)
+void Driver::statusDetail(String& str)
 {
     ObjList* l = m_chans.skipNull();
     for (; l; l=l->skipNext()) {
@@ -1132,6 +1243,36 @@ void Driver::statusChannels(String& str)
     }
 }
 
+bool Driver::commandComplete(Message& msg, const String& partLine, const String& partWord)
+{
+    bool ok = false;
+    bool listChans = String(msg.getValue("complete")) == "channels";
+    if (listChans && (partWord.null() || name().startsWith(partWord)))
+	msg.retValue().append(name(),"\t");
+    else
+	ok = Module::commandComplete(msg,partLine,partWord);
+    lock();
+    unsigned int nchans = m_chans.count();
+    unlock();
+    if (nchans && listChans) {
+	if (name().startsWith(partWord)) {
+	    msg.retValue().append(prefix(),"\t");
+	    return ok;
+	}
+	if (partWord.startsWith(prefix()))
+	    ok = true;
+	lock();
+	ObjList* l = m_chans.skipNull();
+	for (; l; l=l->skipNext()) {
+	    Channel* c = static_cast<Channel*>(l->get());
+	    if (c->id().startsWith(partWord))
+		msg.retValue().append(c->id(),"\t");
+	}
+	unlock();
+    }
+    return ok;
+}
+
 bool Driver::setDebug(Message& msg, const String& target)
 {
     if (!target.startsWith(m_prefix))
