@@ -1661,6 +1661,15 @@ bool Client::callStart(const String& target, const String& line,
 	m->setParam("protocol",proto);
     if (account)
 	m->setParam("account",account);
+    String tmp;
+    if (getText("def_username",tmp) && tmp)
+	m->setParam("caller",tmp);
+    tmp.clear();
+    if (getText("def_callerid",tmp) && tmp)
+	m->setParam("callername",tmp);
+    tmp.clear();
+    if (getText("def_domain",tmp) && tmp)
+	m->setParam("domain",tmp);
     return cc->startRouter(m);
 }
 
@@ -2100,6 +2109,8 @@ ClientChannel::~ClientChannel()
     closeMedia();
     String tmp("Hung up:");
     tmp << " " << (address() ? address() : id());
+    if (m_reason)
+	tmp << " reason: " << m_reason;
     if (Client::self()) {
 	Client::self()->delChannel(this);
 	Client::self()->setStatusLocked(tmp);
@@ -2107,6 +2118,13 @@ ClientChannel::~ClientChannel()
     Engine::enqueue(message("chan.hangup"));
 }
 
+void ClientChannel::disconnected(bool final, const char* reason)
+{
+    Channel::disconnected(final,reason);
+    if (!final)
+	m_reason = reason;
+}
+
 bool ClientChannel::openMedia(bool replace)
 {
     String dev = ClientDriver::device();
@@ -2114,6 +2132,7 @@ bool ClientChannel::openMedia(bool replace)
 	return false;
     if ((!replace) && getSource() && getConsumer())
 	return true;
+    Debug(this,DebugAll,"Opening media channels [%p]",this);
     Message m("chan.attach");
     complete(m,true);
     m.setParam("source",dev);
@@ -2169,7 +2188,7 @@ void ClientChannel::update(bool client)
     }
     desc << " " << id();
     m_desc = desc;
-    XDebug(ClientDriver::self(),DebugAll,"update %d '%s'",client,desc.c_str());
+    XDebug(this,DebugAll,"update %d '%s'",client,desc.c_str());
     if (client && Client::self())
 	Client::self()->setChannel(this);
 }
@@ -2198,7 +2217,7 @@ bool ClientChannel::callRouted(Message& msg)
 
 void ClientChannel::callAccept(Message& msg)
 {
-    Debug(ClientDriver::self(),DebugAll,"ClientChannel::callAccept() [%p]",this);
+    Debug(this,DebugAll,"ClientChannel::callAccept() [%p]",this);
     Client::self()->setStatusLocked("Calling target");
     Channel::callAccept(msg);
     update();
@@ -2224,9 +2243,11 @@ void ClientChannel::callRejected(const char* error, const char* reason, const Me
 
 bool ClientChannel::msgProgress(Message& msg)
 {
-    Debug(ClientDriver::self(),DebugAll,"ClientChannel::msgProgress() [%p]",this);
+    Debug(this,DebugAll,"ClientChannel::msgProgress() [%p]",this);
     Client::self()->setStatusLocked("Call progressing");
-    CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+    CallEndpoint* ch = getPeer();
+    if (!ch)
+	ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
     if (ch && ch->getSource())
 	openMedia();
     bool ret = Channel::msgProgress(msg);
@@ -2236,9 +2257,11 @@ bool ClientChannel::msgProgress(Message& msg)
 
 bool ClientChannel::msgRinging(Message& msg)
 {
-    Debug(ClientDriver::self(),DebugAll,"ClientChannel::msgRinging() [%p]",this);
+    Debug(this,DebugAll,"ClientChannel::msgRinging() [%p]",this);
     Client::self()->setStatusLocked("Call ringing");
-    CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+    CallEndpoint* ch = getPeer();
+    if (!ch)
+	ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
     if (ch && ch->getSource())
 	openMedia();
     bool ret = Channel::msgRinging(msg);
@@ -2248,12 +2271,13 @@ bool ClientChannel::msgRinging(Message& msg)
 
 bool ClientChannel::msgAnswered(Message& msg)
 {
-    Debug(ClientDriver::self(),DebugAll,"ClientChannel::msgAnswered() [%p]",this);
+    Debug(this,DebugAll,"ClientChannel::msgAnswered() [%p]",this);
     m_time = Time::now();
     m_flashing = true;
     m_canAnswer = false;
     m_canConference = true;
     m_canTransfer = true;
+    m_reason.clear();
     Client::self()->setStatusLocked("Call answered");
     openMedia();
     bool ret = Channel::msgAnswered(msg);
@@ -2263,12 +2287,13 @@ bool ClientChannel::msgAnswered(Message& msg)
 
 void ClientChannel::callAnswer()
 {
-    Debug(ClientDriver::self(),DebugAll,"ClientChannel::callAnswer() [%p]",this);
+    Debug(this,DebugAll,"ClientChannel::callAnswer() [%p]",this);
     m_time = Time::now();
     m_flashing = false;
     m_canAnswer = false;
     m_canConference = true;
     m_canTransfer = true;
+    m_reason.clear();
     status("answered");
     Client::self()->setStatus("Call answered");
     openMedia();
