@@ -40,6 +40,8 @@ static const FormatInfo s_formats[] = {
     FormatInfo("2*slin", 320, 10000, "audio", 8000, 2),
     FormatInfo("2*slin/16000", 640, 10000, "audio", 16000, 2),
     FormatInfo("2*slin/32000", 1280, 10000, "audio", 32000, 2),
+    FormatInfo("2*alaw", 160, 10000, "audio", 8000, 2),
+    FormatInfo("2*mulaw", 160, 10000, "audio", 8000, 2),
     FormatInfo("gsm", 33, 20000),
     FormatInfo("ilbc20", 38, 20000),
     FormatInfo("ilbc30", 50, 30000),
@@ -58,6 +60,12 @@ static TranslatorCaps s_simpleCaps[] = {
     { s_formats+1, s_formats+2, 1 },
     { s_formats+2, s_formats+0, 1 },
     { s_formats+2, s_formats+1, 1 },
+    { s_formats+9, s_formats+12, 1 },
+    { s_formats+9, s_formats+13, 1 },
+    { s_formats+12, s_formats+9, 1 },
+    { s_formats+13, s_formats+9, 1 },
+    { s_formats+12, s_formats+13, 1 },
+    { s_formats+13, s_formats+12, 1 },
     { 0, 0, 0 }
 };
 
@@ -145,9 +153,19 @@ public:
 	{
 	    if (!ref())
 		return;
-	    if (getTransSource()) {
+	    while (getTransSource()) {
+		int nchan = m_format.numChannels();
+		if (nchan != getTransSource()->getFormat().numChannels())
+		    break;
+		String sFmt = m_format;
+		String dFmt = getTransSource()->getFormat();
+		if (nchan != 1) {
+		    // get rid of the channel prefix
+		    sFmt >> "*";
+		    dFmt >> "*";
+		}
 		DataBlock oblock;
-		if (oblock.convert(data, m_format, getTransSource()->getFormat())) {
+		if (oblock.convert(data, sFmt, dFmt)) {
 		    if (tStamp == (unsigned long)-1) {
 			unsigned int delta = data.length();
 			if (delta > oblock.length())
@@ -157,6 +175,7 @@ public:
 		    m_timestamp = tStamp;
 		    getTransSource()->Forward(oblock, tStamp);
 		}
+		break;
 	    }
 	    deref();
 	}
@@ -178,16 +197,17 @@ public:
 	    if (!n || (n & 1) || !m_sRate || !m_dRate || !ref())
 		return;
 	    n /= 2;
-	    if (getTransSource()) {
+	    DataSource* src = getTransSource();
+	    if (src) {
 		long delta = tStamp - m_timestamp;
-		unsigned short* s = (unsigned short*) data.data();
+		short* s = (short*) data.data();
 		DataBlock oblock;
 		if (m_dRate > m_sRate) {
 		    int mul = m_dRate / m_sRate;
 		    // repeat the sample an integer number of times
 		    delta *= mul;
-		    oblock.assign(0,n*mul);
-		    unsigned short* d = (unsigned short*) oblock.data();
+		    oblock.assign(0,2*n*mul);
+		    short* d = (short*) oblock.data();
 		    while (n--) {
 			// TODO: smooth the data a little
 			short v = *s++;
@@ -199,8 +219,9 @@ public:
 		    int div = m_sRate / m_dRate;
 		    // average an integer number of samples
 		    delta /= div;
-		    oblock.assign(0,n/div);
-		    unsigned short* d = (unsigned short*) oblock.data();
+		    n /= div;
+		    oblock.assign(0,2*n);
+		    short* d = (short*) oblock.data();
 		    while (n--) {
 			// TODO: interpolate
 			int v = 0;
@@ -215,7 +236,9 @@ public:
 			*d++ = v;
 		    }
 		}
-		getTransSource()->Forward(oblock, m_timestamp + delta);
+		if (src->timeStamp() != invalidStamp())
+		    delta += src->timeStamp();
+		src->Forward(oblock, delta);
 	    }
 	    deref();
 	}
@@ -244,8 +267,10 @@ public:
 		    oblock.assign(0,n*4);
 		    unsigned short* d = (unsigned short*) oblock.data();
 		    // duplicate the sample for each channel
-		    while (n--)
-			*d++ = *d++ = *s++; // valid - in case you wonder
+		    while (n--) {
+			unsigned short v = *d++ = *s++;
+			*d++ = v;
+		    }
 		}
 		else if ((m_sChans == 2) && (m_dChans == 1)) {
 		    oblock.assign(0,n);
@@ -307,7 +332,10 @@ public:
     virtual void removed(const TranslatorFactory* factory);
     virtual DataTranslator* create(const DataFormat& sFormat, const DataFormat& dFormat);
     virtual const TranslatorCaps* getCapabilities() const
-	{ return m_capabilities; }
+	{
+	    return (m_factory1 && m_factory1->getCapabilities() &&
+		m_factory2 && m_factory2->getCapabilities()) ? m_capabilities : 0;
+	}
     virtual unsigned int length() const
 	{ return m_length; }
     virtual const FormatInfo* intermediate() const;
@@ -908,6 +936,8 @@ void DataEndpoint::clearSniffers()
 
 ThreadedSource::~ThreadedSource()
 {
+    if (m_asyncDelete && m_thread)
+	Debug(DebugFail,"ThreadedSource destroyed holding thread %p [%p]",m_thread,this);
     stop();
 }
 
@@ -941,6 +971,16 @@ void ThreadedSource::stop()
 
 void ThreadedSource::cleanup()
 {
+    if (m_asyncDelete && !alive())
+	delete this;
+}
+
+void ThreadedSource::zeroRefs()
+{
+    // let the data thread destroy us if possible
+    if (m_asyncDelete && m_thread && m_thread->running())
+	return;
+    DataSource::zeroRefs();
 }
 
 Thread* ThreadedSource::thread() const
@@ -948,6 +988,11 @@ Thread* ThreadedSource::thread() const
     return m_thread;
 }
 
+bool ThreadedSource::running() const
+{
+    return m_thread && m_thread->running();
+}
+
 
 DataTranslator::DataTranslator(const char* sFormat, const char* dFormat)
     : DataConsumer(sFormat)
@@ -1191,6 +1236,82 @@ ObjList* DataTranslator::destFormats(const DataFormat& sFormat, int maxCost, uns
     return lst;
 }
 
+// helper function to avoid duplicating large amounts of code
+static void mergeOne(ObjList*& lst, const ObjList* formats, const DataFormat& fmt, const FormatInfo* fi, bool sameRate, bool sameChans)
+{
+    if (!fi)
+	return;
+    const DataFormat fmti(fi);
+    if (lst && lst->find(fmti))
+	return;
+    if (formats->find(fmti))
+	return;
+    const FormatInfo* fo = fmt.getInfo();
+    if (fo == fi)
+	return;
+    if (sameRate && (fo->sampleRate != fi->sampleRate))
+	return;
+    if (sameChans && (fo->numChannels != fi->numChannels))
+	return;
+    if (DataTranslator::canConvert(fmt,fmti)) {
+	if (!lst)
+	    lst = new ObjList;
+	lst->append(new String(fmti));
+    }
+}
+
+ObjList* DataTranslator::allFormats(const ObjList* formats, bool existing, bool sameRate, bool sameChans)
+{
+    if (!formats)
+	return 0;
+    ObjList* lst = 0;
+    s_mutex.lock();
+    compose();
+    const ObjList* fmts;
+    if (existing) {
+	// put existing formats first
+	for (fmts = formats; fmts; fmts = fmts->next()) {
+	    const String* fmt = static_cast<const String*>(fmts->get());
+	    if (!(fmt && *fmt))
+		continue;
+	    const FormatInfo* fo = FormatRepository::getFormat(*fmt);
+	    if (fo) {
+		if (!lst)
+		    lst = new ObjList;
+		lst->append(new String(fo->name));
+	    }
+	}
+    }
+    for (fmts = formats; fmts; fmts = fmts->next()) {
+	const String* fmt = static_cast<const String*>(fmts->get());
+	if (!(fmt && *fmt))
+	    continue;
+	const FormatInfo* fo = FormatRepository::getFormat(*fmt);
+	if (!fo)
+	    continue;
+	const DataFormat fmto(fo);
+
+	// search in the static list first
+	for (unsigned int i = 0; i < (sizeof(s_formats)/sizeof(FormatInfo)); i++)
+	    mergeOne(lst,formats,fmto,s_formats+i,sameRate,sameChans);
+	// then try the installed formats
+	for (flist* l = s_flist; l; l = l->next)
+	    mergeOne(lst,formats,fmto,l->info,sameRate,sameChans);
+    }
+    s_mutex.unlock();
+    return lst;
+}
+
+ObjList* DataTranslator::allFormats(const String& formats, bool existing, bool sameRate, bool sameChans)
+{
+    ObjList* fmts = formats.split(',',false);
+    if (!fmts)
+	return 0;
+    ObjList* lst = allFormats(fmts,existing,sameRate,sameChans);
+    delete fmts;
+    return lst;
+}
+
 bool DataTranslator::canConvert(const DataFormat& fmt1, const DataFormat& fmt2)
 {
     if (fmt1 == fmt2)
@@ -1471,8 +1592,10 @@ DataTranslator* ChainedFactory::create(const DataFormat& sFormat, const DataForm
     if (trans2) {
 	XDebug(DebugInfo,"Chaining translators: '%s' %p --(%s)-> %p '%s' [%p]",
 	    sFormat.c_str(),trans,m_format.c_str(),trans2,dFormat.c_str(),this);
-	trans->getTransSource()->attach(trans2);
-	trans2->deref();
+	// trans2 may be a chain itself so find the first translator
+	DataTranslator* trans1 = trans2->getFirstTranslator();
+	trans->getTransSource()->attach(trans1);
+	trans1->deref();
     }
     else
 	trans->destruct();
