@@ -96,6 +96,7 @@ using namespace TelEngine;
 
 #define MAX_SANITY 5
 #define INIT_SANITY 10
+#define MAX_LOGBUFF 4096
 
 static u_int64_t s_nextinit = 0;
 static u_int64_t s_restarts = 0;
@@ -138,6 +139,7 @@ String Engine::s_modpath(MOD_PATH);
 String Engine::s_extramod;
 String Engine::s_modsuffix(DLL_SUFFIX);
 
+Engine::RunMode Engine::s_mode = Engine::Stopped;
 Engine* Engine::s_self = 0;
 int Engine::s_haltcode = -1;
 int EnginePrivate::count = 0;
@@ -199,7 +201,7 @@ bool EngineStatusHandler::received(Message &msg)
     msg.retValue() << ",workers=" << EnginePrivate::count;
     msg.retValue() << ",mutexes=" << Mutex::count();
     msg.retValue() << ",locks=" << Mutex::locks();
-    msg.retValue() << "\n";
+    msg.retValue() << "\r\n";
     return false;
 }
 
@@ -300,7 +302,7 @@ static void serviceMain(DWORD argc, LPTSTR* argv)
     s_status.dwWaitHint = 0;
     s_handler = ::RegisterServiceCtrlHandler("yate",serviceHandler);
     if (!s_handler) {
-	Debug(DebugFail,"Could not register service control handler \"yate\", code %u\n",::GetLastError());
+	Debug(DebugFail,"Could not register service control handler \"yate\", code %u",::GetLastError());
 	return;
     }
     setStatus(SERVICE_START_PENDING);
@@ -318,12 +320,21 @@ static SERVICE_TABLE_ENTRY dispatchTable[] =
 #define setStatus(s)
 #define checkPoint()
 
+static bool s_logrotator = false;
 static bool s_runagain = true;
 static pid_t s_childpid = -1;
+static pid_t s_superpid = -1;
 
 static void superhandler(int signal)
 {
     switch (signal) {
+	case SIGHUP:
+	    if (s_logrotator) {
+		::fprintf(stderr,"Supervisor (%d) closing the log file\n",s_superpid);
+		logFileOpen();
+		::fprintf(stderr,"Supervisor (%d) reopening the log file\n",s_superpid);
+	    }
+	    break;
 	case SIGINT:
 	case SIGTERM:
 	case SIGABRT:
@@ -333,9 +344,21 @@ static void superhandler(int signal)
 	::kill(s_childpid,signal);
 }
 
+static void copystream(int dest, int src)
+{
+    for (;;) {
+	char buf[MAX_LOGBUFF];
+	int rd = ::read(src,buf,sizeof(buf));
+	if (rd <= 0)
+	    break;
+	::write(dest,buf,rd);
+    }
+}
+
 static int supervise(void)
 {
-    ::fprintf(stderr,"Supervisor (%u) is starting\n",::getpid());
+    s_superpid = ::getpid();
+    ::fprintf(stderr,"Supervisor (%u) is starting\n",s_superpid);
     ::signal(SIGINT,superhandler);
     ::signal(SIGTERM,superhandler);
     ::signal(SIGHUP,superhandler);
@@ -348,11 +371,20 @@ static int supervise(void)
 	int wdogfd[2];
 	if (::pipe(wdogfd)) {
 	    int err = errno;
-	    ::fprintf(stderr,"Supervisor: pipe failed: %s (%d)\n",::strerror(err),err);
+	    ::fprintf(stderr,"Supervisor: watchdog pipe failed: %s (%d)\n",::strerror(err),err);
 	    return err;
 	}
 	::fcntl(wdogfd[0],F_SETFL,O_NONBLOCK);
 	::fcntl(wdogfd[1],F_SETFL,O_NONBLOCK);
+	int logfd[2] = { -1, -1 };
+	if (s_logrotator) {
+	    if (::pipe(logfd)) {
+		int err = errno;
+		::fprintf(stderr,"Supervisor: log pipe failed: %s (%d)\n",::strerror(err),err);
+		return err;
+	    }
+	    ::fcntl(logfd[0],F_SETFL,O_NONBLOCK);
+	}
 	s_childpid = ::fork();
 	if (s_childpid < 0) {
 	    int err = errno;
@@ -362,6 +394,15 @@ static int supervise(void)
 	if (s_childpid == 0) {
 	    s_super_handle = wdogfd[1];
 	    ::close(wdogfd[0]);
+	    if (s_logrotator) {
+		::close(logfd[0]);
+		// Redirect stdout and stderr to the new file
+		::fflush(stdout);
+		::dup2(logfd[1],1);
+		::fflush(stderr);
+		::dup2(logfd[1],2);
+		::close(logfd[1]);
+	    }
 	    ::signal(SIGINT,SIG_DFL);
 	    ::signal(SIGTERM,SIG_DFL);
 	    ::signal(SIGHUP,SIG_DFL);
@@ -370,6 +411,8 @@ static int supervise(void)
 	    return -1;
 	}
 	::close(wdogfd[1]);
+	if (s_logrotator)
+	    ::close(logfd[1]);
 	// Wait for the child to die or block
 	for (int sanity = INIT_SANITY; sanity > 0; sanity--) {
 	    int status = -1;
@@ -395,14 +438,19 @@ static int supervise(void)
 	    tmp = ::read(wdogfd[0],buf,sizeof(buf));
 	    if (tmp >= 0) {
 		// Timer messages add one sanity point every second
-		sanity += tmp;
-		if (sanity > MAX_SANITY)
-		    sanity = MAX_SANITY;
+		tmp += sanity;
+		if (tmp > MAX_SANITY)
+		    tmp = MAX_SANITY;
+		if (sanity < tmp)
+		    sanity = tmp;
 	    }
 	    else if ((errno != EINTR) && (errno != EAGAIN))
 		break;
 	    // Consume sanity points slightly slower than added
-	    ::usleep(1200000);
+	    for (int i = 0; i < 12; i++) {
+		copystream(2,logfd[0]);
+		::usleep(100000);
+	    }
 	}
 	::close(wdogfd[0]);
 	if (s_childpid > 0) {
@@ -411,17 +459,21 @@ static int supervise(void)
 	    // If -Da was specified try to get a corefile
 	    if (s_sigabrt) {
 		::kill(s_childpid,SIGABRT);
-		::usleep(250000);
+		::usleep(500000);
 	    }
 	    ::kill(s_childpid,SIGKILL);
 	    ::usleep(10000);
 	    ::waitpid(s_childpid,0,WNOHANG);
 	    s_childpid = -1;
 	}
+	if (s_logrotator) {
+	    copystream(2,logfd[0]);
+	    ::close(logfd[0]);
+	}
 	if (s_runagain)
 	    ::usleep(1000000);
     }
-    ::fprintf(stderr,"Supervisor (%d) exiting with code %d\n",::getpid(),retcode);
+    ::fprintf(stderr,"Supervisor (%d) exiting with code %d\n",s_superpid,retcode);
     return retcode;
 }
 #endif /* _WINDOWS */
@@ -505,6 +557,7 @@ Engine::~Engine()
     m_dispatcher.clear();
     plugins.clear();
     m_libs.clear();
+    s_mode = Stopped;
     s_self = 0;
 }
 
@@ -932,7 +985,9 @@ static void usage(bool client, FILE* f)
 "     w            Delay creation of 1st worker thread\n"
 "     o            Colorize output using ANSI codes\n"
 "     s            Abort on bugs even during shutdown\n"
-"     t            Timestamp debugging messages\n"
+"     t            Timestamp debugging messages relative to program start\n"
+"     e            Timestamp debugging messages based on EPOCH (1-1-1970 GMT)\n"
+"     f            Timestamp debugging in GMT format YYYYMMDDhhmmss.uuuuuu\n"
     ,client ? "" :
 #ifdef _WINDOWS
 "   --service      Run as Windows service\n"
@@ -941,6 +996,7 @@ static void usage(bool client, FILE* f)
 #else
 "   -d             Daemonify, suppress output unless logged\n"
 "   -s             Supervised, restart if crashes or locks up\n"
+"   -r             Enable rotation of log file (needs -s and -l)\n"
 #endif
     ,s_cfgfile.safe());
 }
@@ -974,7 +1030,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
     bool supervised = false;
 #endif
     bool client = (mode == Client);
-    bool tstamp = false;
+    Debugger::Formatting tstamp = Debugger::None;
     bool colorize = false;
     const char* pidfile = 0;
     const char* workdir = 0;
@@ -1046,6 +1102,9 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 		    case 's':
 			supervised = true;
 			break;
+		    case 'r':
+			s_logrotator = true;
+			break;
 #endif
 		    case 'p':
 			if (i+1 >= argc) {
@@ -1135,8 +1194,14 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 				case 'o':
 				    colorize = true;
 				    break;
+				case 'e':
+				    tstamp = Debugger::Absolute;
+				    break;
 				case 't':
-				    tstamp = true;
+				    tstamp = Debugger::Relative;
+				    break;
+				case 'f':
+				    tstamp = Debugger::Textual;
 				    break;
 				default:
 				    badopt(client,*pc,argv[i]);
@@ -1164,6 +1229,8 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
     if (fail)
 	return EINVAL;
 
+    s_mode = mode;
+
     s_cfgfile = cfgfile;
     if (s_cfgfile.endsWith(".exe") || s_cfgfile.endsWith(".EXE"))
 	s_cfgfile = s_cfgfile.substr(0,s_cfgfile.length()-4);
@@ -1240,6 +1307,10 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 	::fprintf(stderr,"Option -Do not supported when logging to file\n");
 	return EINVAL;
     }
+    if (s_logrotator && !(supervised && s_logfile)) {
+	::fprintf(stderr,"Option -r needs supervisor and logging to file\n");
+	return EINVAL;
+    }
     Debugger::enableOutput(true,colorize);
     if (daemonic) {
 	Debugger::enableOutput(false);
@@ -1263,19 +1334,6 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 	}
     }
 
-    if (s_logfile) {
-	int fd = ::open(s_logfile,O_WRONLY|O_CREAT|O_APPEND,0640);
-	if (fd >= 0) {
-	    // Redirect stdout and stderr to the new file
-	    ::fflush(stdout);
-	    ::dup2(fd,1);
-	    ::fflush(stderr);
-	    ::dup2(fd,2);
-	    ::close(fd);
-	    Debugger::enableOutput(true);
-	}
-    }
-
 #ifdef _WINDOWS
     if (!service)
 #endif
@@ -1310,8 +1368,7 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, bo
 	return retcode;
 #endif
 
-    if (tstamp)
-	setDebugTimestamp();
+    Debugger::setFormatting(tstamp);
 
 #ifdef _WINDOWS
     if (service)
