@@ -38,12 +38,16 @@ INIT_PLUGIN(DumbDriver);
 class DumbChannel :  public Channel
 {
 public:
-    DumbChannel(const char* addr = 0) : Channel(__plugin) {
+    DumbChannel(const char* addr = 0, const NamedList* exeMsg = 0) :
+      Channel(__plugin, 0, (exeMsg != 0))
+    {
 	m_address = addr;
-	Engine::enqueue(message("chan.startup"));
+	Engine::enqueue(message("chan.startup",exeMsg));
     };
     ~DumbChannel();
     virtual void disconnected(bool final, const char *reason);
+    inline void setTargetid(const char* targetid)
+	{ m_targetid = targetid; }
 };
 
 void DumbChannel::disconnected(bool final, const char *reason)
@@ -58,15 +62,15 @@ DumbChannel::~DumbChannel()
     Engine::enqueue(message("chan.hangup"));
 }
 
-
 bool DumbDriver::msgExecute(Message& msg, String& dest)
 {
     CallEndpoint *dd = static_cast<CallEndpoint *>(msg.userData());
     if (dd) {
-	DumbChannel *c = new DumbChannel(dest);
+	DumbChannel *c = new DumbChannel(dest,&msg);
 	if (dd->connect(c)) {
 	    msg.setParam("peerid", c->id());
 	    msg.setParam("targetid", c->id());
+	    c->setTargetid(dd->id());
 	    // autoring unless parameter is already set in message
 	    if (!msg.getParam("autoring"))
 		msg.addParam("autoring","true");
@@ -87,29 +91,55 @@ bool DumbDriver::msgExecute(Message& msg, String& dest)
 
     DumbChannel* c = new DumbChannel(dest);
 
+    String caller = msg.getValue("caller");
+    if (caller.null())
+	caller << prefix() << dest;
+
     Message m("call.route");
     m.addParam("driver","dumb");
     m.addParam("id", c->id());
-    m.addParam("caller",dest);
+    m.addParam("caller",caller);
     m.addParam("called",targ);
+    m.copyParam(msg,"callername");
+    m.copyParam(msg,"maxcall");
+    m.copyParam(msg,"timeout");
+
+    String params = msg.getValue("copyparams");
+    if (params) {
+	ObjList* lst = params.split(',',false);
+	for (ObjList* l = lst; l; l = l->next()) {
+	    String* s = static_cast<String*>(l->get());
+	    if (!s)
+		continue;
+	    s->trimBlanks();
+	    if (*s)
+		m.copyParam(msg,*s);
+	}
+	delete lst;
+    }
 
     if (Engine::dispatch(m)) {
 	m = "call.execute";
 	m.addParam("callto",m.retValue());
-	if (msg.getParam("maxcall"))
-	    m.setParam("maxcall", msg.getValue("maxcall"));
 	m.retValue().clear();
 	m.setParam("id", c->id());
 	m.userData(c);
-	if (Engine::dispatch(m)) {
+	if (Engine::dispatch(m) && c->callRouted(m)) {
+	    c->callAccept(m);
+	    msg.copyParam(m,"id");
+	    msg.copyParam(m,"peerid");
+	    const char* targetid = m.getValue("targetid");
+	    if (targetid) {
+		msg.setParam("targetid",targetid);
+		c->setTargetid(targetid);
+	    }
 	    c->deref();
-	    msg.setParam("id", m.getValue("id"));
-	    msg.setParam("targetid", m.getValue("targetid"));
-	    msg.setParam("peerid", m.getValue("peerid"));
 	    return true;
 	}
-	else if (m.getParam("reason"))
-	    msg.setParam("reason", m.getValue("reason"));
+	else {
+	    msg.copyParam(m,"error");
+	    msg.copyParam(m,"reason");
+	}
 	Debug(this,DebugWarn,"Outgoing call not accepted!");
     }
     else
