@@ -88,6 +88,7 @@ namespace { // anonymous
 static bool s_externalRtp;
 static bool s_fallbackRtp;
 static bool s_passtrough;
+static bool s_pwlibThread;
 static int s_maxCleaning = 0;
 
 static Configuration s_cfg;
@@ -223,7 +224,7 @@ class H323Process : public PProcess
 	    s_cfg.getValue("general","product","YATE"),
 	    (unsigned short)s_cfg.getIntValue("general","major",YATE_MAJOR),
 	    (unsigned short)s_cfg.getIntValue("general","minor",YATE_MINOR),
-	    (PProcess::CodeStatus)s_cfg.getIntValue("general","status",dict_str2code,PProcess::AlphaCode),
+	    (PProcess::CodeStatus)s_cfg.getIntValue("general","status",dict_str2code,PProcess::ReleaseCode),
 	    (unsigned short)s_cfg.getIntValue("general","build",YATE_BUILD)
 	    )
 	{ Resume(); }
@@ -368,6 +369,7 @@ public:
     ~YateH323Connection();
     virtual H323Connection::AnswerCallResponse OnAnswerCall(const PString& caller,
 	const H323SignalPDU& signalPDU, H323SignalPDU& connectPDU);
+    virtual H323Connection::CallEndReason SendSignalSetup(const PString& alias, const H323TransportAddress& address);
     virtual void OnEstablished();
     virtual void OnCleared();
     virtual BOOL OnAlerting(const H323SignalPDU& alertingPDU, const PString& user);
@@ -394,12 +396,14 @@ public:
     void setCallerID(const char* number, const char* name);
     void rtpExecuted(Message& msg);
     void rtpForward(Message& msg, bool init = false);
-    void answerCall(AnswerCallResponse response);
+    void answerCall(AnswerCallResponse response, bool autoEarly = false);
     static BOOL decodeCapability(const H323Capability& capability, const char** dataFormat, int* payload = 0, String* capabName = 0);
     inline bool hasRemoteAddress() const
 	{ return m_passtrough && (m_remotePort > 0); }
     inline bool nativeRtp() const
 	{ return m_nativeRtp; }
+    inline void rtpLocal()
+	{ m_passtrough = false; }
 private:
     String m_chanId;
     YateH323Chan* m_chan;
@@ -488,6 +492,22 @@ protected:
     PString m_name;
 };
 
+class YateCallThread : public PThread
+{
+    PCLASSINFO(YateCallThread, PThread);
+public:
+    YateCallThread(YateH323EndPoint* ep, const char* remoteParty, void* userData, int& status)
+	: PThread(10000), m_ep(ep), m_userData(userData), m_remoteParty(remoteParty), m_status(status)
+	{ }
+    virtual void Main();
+    static bool makeCall(YateH323EndPoint* ep, const char* remoteParty, void* userData, bool newThread = false);
+protected:
+    YateH323EndPoint* m_ep;
+    void* m_userData;
+    PString m_remoteParty;
+    int& m_status;
+};
+
 class UserHandler : public MessageHandler
 {
 public:
@@ -925,6 +945,33 @@ void YateH323EndPoint::checkGkClient()
 }
 
 
+// make a call either normally or in a proxy PWlib thread
+bool YateCallThread::makeCall(YateH323EndPoint* ep, const char* remoteParty, void* userData, bool newThread)
+{
+    if (!newThread) {
+	PString token;
+	return ep->MakeCall(remoteParty,token,userData) != 0;
+    }
+    int status = 0;
+    YateCallThread* call = new YateCallThread(ep,remoteParty,userData,status);
+    call->SetAutoDelete();
+    call->Resume();
+    while (!status)
+	Thread::yield();
+    return status > 0;
+}
+
+// the actual method that does the job in the proxy thread
+void YateCallThread::Main()
+{
+    PString token;
+    if (m_ep->MakeCall(m_remoteParty,token,m_userData))
+	m_status = 1;
+    else
+	m_status = -1;
+}
+
+
 YateH323Connection::YateH323Connection(YateH323EndPoint& endpoint,
     H323Transport* transport, unsigned callReference, void* userdata)
     : H323Connection(endpoint,callReference), m_chan(0), m_mutex(0),
@@ -1066,6 +1113,10 @@ H323Connection::AnswerCallResponse YateH323Connection::OnAnswerCall(const PStrin
 	m->addParam("rtp_addr",m_remoteAddr);
 	m->addParam("rtp_port",String(m_remotePort));
     }
+    else if (m_passtrough) {
+	Debug(this,DebugNote,"Disabling RTP forward because of slow start mode [%p]",this);
+	m_passtrough = false;
+    }
     if (m_remoteFormats)
 	m->addParam("formats",m_remoteFormats);
 
@@ -1114,14 +1165,14 @@ void YateH323Connection::rtpForward(Message& msg, bool init)
     }
 }
 
-void YateH323Connection::answerCall(AnswerCallResponse response)
+void YateH323Connection::answerCall(AnswerCallResponse response, bool autoEarly)
 {
     bool media = false;
     if (hasRemoteAddress() && m_rtpPort)
 	media = true;
-    else {
+    else if (autoEarly) {
 	TelEngine::Lock lock(m_mutex);
-	if (m_chan && m_chan->alive() && m_chan->getConsumer() && m_chan->getConsumer()->getConnSource())
+	if (m_chan && m_chan->alive() && m_chan->getPeer() && m_chan->getPeer()->getSource())
 	    media = true;
     }
     // modify responses to indicate we have early media (remote ringing)
@@ -1140,6 +1191,13 @@ void YateH323Connection::answerCall(AnswerCallResponse response)
     AnsweringCall(response);
 }
 
+H323Connection::CallEndReason YateH323Connection::SendSignalSetup(const PString& alias, const H323TransportAddress& address)
+{
+    if (m_chan && m_chan->address().null())
+	m_chan->setAddress(address);
+    return H323Connection::SendSignalSetup(alias,address);
+}
+
 void YateH323Connection::OnEstablished()
 {
     TelEngine::Lock lock(m_mutex);
@@ -1888,7 +1946,7 @@ YateH323Chan::YateH323Chan(YateH323Connection* conn,Message* msg,const char* add
     Debug(this,DebugAll,"YateH323Chan::YateH323Chan(%p,%s) %s [%p]",
 	conn,addr,direction(),this);
     setMaxcall(msg);
-    Message* s = message("chan.startup");
+    Message* s = message("chan.startup",msg);
     if (msg) {
 	s->setParam("caller",msg->getValue("caller"));
 	s->setParam("called",msg->getValue("called"));
@@ -2036,6 +2094,9 @@ bool YateH323Chan::callRouted(Message& msg)
 {
     Channel::callRouted(msg);
     if (m_conn) {
+	// try to disable RTP forwarding earliest possible
+	if (!msg.getBoolValue("rtp_forward"))
+	    m_conn->rtpLocal();
         String s(msg.retValue());
         if (s.startSkip("h323/",false) && s && msg.getBoolValue("redirect") && m_conn->Lock()) {
             Debug(this,DebugAll,"YateH323Chan redirecting to '%s' [%p]",s.c_str(),this);
@@ -2072,7 +2133,7 @@ bool YateH323Chan::msgProgress(Message& msg)
 	return false;
     if (msg.getParam("rtp_forward"))
 	m_conn->rtpForward(msg);
-    m_conn->answerCall(H323Connection::AnswerCallDeferred);
+    m_conn->answerCall(H323Connection::AnswerCallDeferred,msg.getBoolValue("earlymedia",true));
     return true;
 }
 
@@ -2083,7 +2144,7 @@ bool YateH323Chan::msgRinging(Message& msg)
 	return false;
     if (msg.getParam("rtp_forward"))
 	m_conn->rtpForward(msg);
-    m_conn->answerCall(H323Connection::AnswerCallPending);
+    m_conn->answerCall(H323Connection::AnswerCallPending,msg.getBoolValue("earlymedia",true));
     return true;
 }
 
@@ -2210,15 +2271,10 @@ bool H323Driver::msgExecute(Message& msg, String& dest)
     }
     Debug(this,DebugInfo,"Found call to H.323 target='%s'",
 	dest.c_str());
-    PString p;
     YateH323EndPoint* ep = hplugin.findEndpoint(msg.getValue("line"));
     if (ep) {
-	YateH323Connection* conn = static_cast<YateH323Connection*>(
-	    ep->MakeCallLocked(dest.c_str(),p,&msg));
-	if (conn) {
-	    conn->Unlock();
+	if (YateCallThread::makeCall(ep,dest.c_str(),&msg,msg.getBoolValue("pwlibthread",s_pwlibThread)))
 	    return true;
-	}
 	// the only reason a YateH323Connection is not created is congestion
 	msg.setParam("error","congestion");
 	return false;
@@ -2254,6 +2310,7 @@ void H323Driver::initialize()
     // mantain compatibility with old config files
     s_passtrough = s_cfg.getBoolValue("general","passtrough_rtp",s_passtrough);
     s_maxCleaning = s_cfg.getIntValue("general","maxcleaning",100);
+    s_pwlibThread = s_cfg.getBoolValue("general","pwlibthread");
     maxRoute(s_cfg.getIntValue("incoming","maxqueue",5));
     maxChans(s_cfg.getIntValue("ep","maxconns",0));
     if (!s_process) {
