@@ -101,6 +101,7 @@ public:
     static Tone* buildDtmf(const String& dtmf, int len = DTMF_LEN, int gap = DTMF_GAP);
 protected:
     ToneSource(const ToneDesc* tone = 0);
+    virtual void zeroRefs();
     String m_name;
     const Tone* m_tone;
     int m_repeat;
@@ -258,6 +259,18 @@ static const ToneDesc s_desc[] = {
     { 0, 0, 0 }
 };
 
+// This function is here mainly to avoid 64bit gcc b0rking optimizations
+static unsigned int byteRate(u_int64_t time, unsigned int bytes)
+{
+    if (!(time && bytes))
+	return 0;
+    time = Time::now() - time;
+    if (!time)
+	return 0;
+    return (unsigned int)((bytes*(u_int64_t)1000000 + time/2) / time);
+}
+
+
 ToneData::ToneData(const char* desc)
     : m_f1(0), m_f2(0), m_mod(false), m_data(0)
 {
@@ -383,6 +396,7 @@ ToneData* ToneData::getData(const char* desc)
     return d;
 }
 
+
 ToneSource::ToneSource(const ToneDesc* tone)
     : m_tone(0), m_repeat(tone == 0),
       m_data(0,320), m_brate(16000), m_total(0), m_time(0)
@@ -393,21 +407,24 @@ ToneSource::ToneSource(const ToneDesc* tone)
     }
     Debug(&__plugin,DebugAll,"ToneSource::ToneSource(%p) '%s' [%p]",
 	tone,m_name.c_str(),this);
+    asyncDelete(true);
 }
 
 ToneSource::~ToneSource()
 {
     Lock lock(__plugin);
     Debug(&__plugin,DebugAll,"ToneSource::~ToneSource() [%p] total=%u stamp=%lu",this,m_total,timeStamp());
-    tones.remove(this,false);
     stop();
-    if (m_time) {
-	m_time = Time::now() - m_time;
-	if (m_time) {
-	    m_time = (m_total*(u_int64_t)1000000 + m_time/2) / m_time;
-	    Debug(&__plugin,DebugInfo,"ToneSource rate=" FMT64U " b/s",m_time);
-	}
-    }
+    if (m_time)
+	Debug(&__plugin,DebugInfo,"ToneSource rate=%u b/s",byteRate(m_time,m_total));
+}
+
+void ToneSource::zeroRefs()
+{
+    __plugin.lock();
+    tones.remove(this,false);
+    __plugin.unlock();
+    ThreadedSource::zeroRefs();
 }
 
 bool ToneSource::startup()
@@ -547,12 +564,12 @@ void ToneSource::run()
 	m_total += m_data.length();
 	tpos += (m_data.length()*(u_int64_t)1000000/m_brate);
     }
-    m_time = Time::now() - m_time;
-    m_time = (m_total*(u_int64_t)1000000 + m_time/2) / m_time;
-    Debug(&__plugin,DebugAll,"ToneSource [%p] end, total=%u (" FMT64U " b/s)",this,m_total,m_time);
+    Debug(&__plugin,DebugAll,"ToneSource [%p] end, total=%u (%u b/s)",
+	this,m_total,byteRate(m_time,m_total));
     m_time = 0;
 }
 
+
 TempSource::TempSource(String& desc)
     : m_single(0)
 {
@@ -604,6 +621,7 @@ void TempSource::cleanup()
     deref();
 }
 
+
 ToneChan::ToneChan(String& tone)
     : Channel(__plugin)
 {
@@ -626,6 +644,85 @@ ToneChan::~ToneChan()
     Debug(this,DebugAll,"ToneChan::~ToneChan() %s [%p]",id().c_str(),this);
 }
 
+
+bool AttachHandler::received(Message& msg)
+{
+    String src(msg.getValue("source"));
+    if (!src.startSkip("tone/",false))
+	src.clear();
+    String ovr(msg.getValue("override"));
+    if (!ovr.startSkip("tone/",false))
+	ovr.clear();
+    String repl(msg.getValue("replace"));
+    if (!repl.startSkip("tone/",false))
+	repl.clear();
+    if (src.null() && ovr.null() && repl.null())
+	return false;
+
+    DataEndpoint* de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
+    if (!de) {
+	CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
+	if (ch)
+	    de = ch->setEndpoint();
+    }
+
+    if (!de) {
+	Debug(DebugWarn,"Tone attach request with no control or data channel!");
+	return false;
+    }
+
+    // if single attach was requested we can return true if everything is ok
+    bool ret = msg.getBoolValue("single");
+
+    Lock lock(__plugin);
+    if (src) {
+	ToneSource* t = ToneSource::getTone(src);
+	if (t) {
+	    de->setSource(t);
+	    t->deref();
+	    msg.clearParam("source");
+	}
+	else {
+	    Debug(DebugWarn,"No source tone '%s' could be attached to %p",src.c_str(),de);
+	    ret = false;
+	}
+    }
+    if (ovr) {
+	DataConsumer* c = de->getConsumer();
+	if (c) {
+	    TempSource* t = new TempSource(ovr);
+	    if (DataTranslator::attachChain(t,c,true) && t->startup())
+		msg.clearParam("override");
+	    else {
+		Debug(DebugWarn,"Override source tone '%s' failed to start [%p]",ovr.c_str(),t);
+		ret = false;
+	    }
+	}
+	else {
+	    Debug(DebugWarn,"Requested override '%s' to missing consumer of %p",ovr.c_str(),de);
+	    ret = false;
+	}
+    }
+    if (repl) {
+	DataConsumer* c = de->getConsumer();
+	if (c) {
+	    TempSource* t = new TempSource(repl);
+	    if (DataTranslator::attachChain(t,c,false) && t->startup())
+		msg.clearParam("replace");
+	    else {
+		Debug(DebugWarn,"Replacement source tone '%s' failed to start [%p]",repl.c_str(),t);
+		ret = false;
+	    }
+	}
+	else {
+	    Debug(DebugWarn,"Requested replacement '%s' to missing consumer of %p",repl.c_str(),de);
+	    ret = false;
+	}
+    }
+    return ret;
+}
+
+
 bool ToneGenDriver::msgExecute(Message& msg, String& dest)
 {
     CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userData());
@@ -679,64 +776,6 @@ bool ToneGenDriver::msgExecute(Message& msg, String& dest)
     return true;
 }
 
-bool AttachHandler::received(Message& msg)
-{
-    String src(msg.getValue("source"));
-    if (!src.startSkip("tone/",false))
-	src.clear();
-    String ovr(msg.getValue("override"));
-    if (!ovr.startSkip("tone/",false))
-	ovr.clear();
-    if (src.null() && ovr.null())
-	return false;
-
-    DataEndpoint* de = static_cast<DataEndpoint*>(msg.userObject("DataEndpoint"));
-    if (!de) {
-	CallEndpoint* ch = static_cast<CallEndpoint*>(msg.userObject("CallEndpoint"));
-	if (ch)
-	    de = ch->setEndpoint();
-    }
-
-    if (!de) {
-	Debug(DebugWarn,"Tone attach request with no control or data channel!");
-	return false;
-    }
-
-    // if single attach was requested we can return true if everything is ok
-    bool ret = msg.getBoolValue("single");
-
-    Lock lock(__plugin);
-    if (src) {
-	ToneSource* t = ToneSource::getTone(src);
-	if (t) {
-	    de->setSource(t);
-	    t->deref();
-	    msg.clearParam("source");
-	}
-	else {
-	    Debug(DebugWarn,"No source tone '%s' could be attached to %p",src.c_str(),de);
-	    ret = false;
-	}
-    }
-    if (ovr) {
-	DataConsumer* c = de->getConsumer();
-	if (c) {
-	    TempSource* t = new TempSource(ovr);
-	    if (DataTranslator::attachChain(t,c,true) && t->startup())
-		msg.clearParam("override");
-	    else {
-		Debug(DebugWarn,"Override source tone '%s' failed to start [%p]",ovr.c_str(),t);
-		ret = false;
-	    }
-	}
-	else {
-	    Debug(DebugWarn,"Requested override '%s' to missing consumer of %p",ovr.c_str(),de);
-	    ret = false;
-	}
-    }
-    return ret;
-}
-
 void ToneGenDriver::statusModule(String& str)
 {
     Module::statusModule(str);
