@@ -36,22 +36,25 @@ namespace { // anonymous
 class WaveSource : public ThreadedSource
 {
 public:
-    WaveSource(const String& file, CallEndpoint* chan, bool autoclose = true);
+    static WaveSource* create(const String& file, CallEndpoint* chan, bool autoclose = true, bool autorepeat = false);
     ~WaveSource();
     virtual void run();
     virtual void cleanup();
     void setNotify(const String& id);
 private:
+    WaveSource(const char* file, CallEndpoint* chan, bool autoclose);
+    void init(const String& file, bool autorepeat);
     void detectAuFormat();
     void detectWavFormat();
     void detectIlbcFormat();
     bool computeDataRate();
-    bool notify();
+    bool notify(DataSource* source, const char* reason = 0);
     CallEndpoint* m_chan;
     DataBlock m_data;
     int m_fd;
     bool m_swap;
     unsigned m_brate;
+    long m_repeatPos;
     unsigned m_total;
     u_int64_t m_time;
     String m_id;
@@ -63,14 +66,25 @@ private:
 class WaveConsumer : public DataConsumer
 {
 public:
-    WaveConsumer(const String& file, CallEndpoint* chan = 0, unsigned maxlen = 0);
+    enum Header {
+	None = 0,
+	Au,
+	Ilbc,
+    };
+    WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxlen, const char* format = 0);
     ~WaveConsumer();
+    virtual bool setFormat(const DataFormat& format);
     virtual void Consume(const DataBlock& data, unsigned long tStamp);
     inline void setNotify(const String& id)
 	{ m_id = id; }
 private:
+    void writeIlbcHeader() const;
+    void writeAuHeader();
     CallEndpoint* m_chan;
     int m_fd;
+    bool m_swap;
+    bool m_locked;
+    Header m_header;
     unsigned m_total;
     unsigned m_maxlen;
     u_int64_t m_time;
@@ -80,21 +94,21 @@ private:
 class WaveChan : public Channel
 {
 public:
-    WaveChan(const String& file, bool record, unsigned maxlen = 0);
+    WaveChan(const String& file, bool record, unsigned maxlen, bool autorepeat, const char* format = 0);
     ~WaveChan();
 };
 
 class Disconnector : public Thread
 {
 public:
-    Disconnector(CallEndpoint* chan, const String& id, bool source, bool disc);
+    Disconnector(CallEndpoint* chan, const String& id, DataSource* source, bool disc, const char* reason = 0);
     virtual ~Disconnector();
     virtual void run();
     bool init();
 private:
     RefPointer<CallEndpoint> m_chan;
     Message* m_msg;
-    bool m_source;
+    DataSource* m_source;
     bool m_disc;
 };
 
@@ -122,14 +136,32 @@ private:
     AttachHandler* m_handler;
 };
 
+bool s_asyncDelete = true;
+
 INIT_PLUGIN(WaveFileDriver);
 
-WaveSource::WaveSource(const String& file, CallEndpoint* chan, bool autoclose)
-    : m_chan(chan), m_fd(-1), m_swap(false), m_brate(0),
-      m_total(0), m_time(0), m_autoclose(autoclose), m_autoclean(false),
-      m_nodata(false)
+
+typedef struct {
+    uint32_t sign;
+    uint32_t offs;
+    uint32_t len;
+    uint32_t form;
+    uint32_t freq;
+    uint32_t chan;
+} AuHeader;
+
+#define ILBC_HEADER_LEN 9
+
+
+WaveSource* WaveSource::create(const String& file, CallEndpoint* chan, bool autoclose, bool autorepeat)
+{
+    WaveSource* tmp = new WaveSource(file,chan,autoclose);
+    tmp->init(file,autorepeat);
+    return tmp;
+}
+
+void WaveSource::init(const String& file, bool autorepeat)
 {
-    Debug(&__plugin,DebugAll,"WaveSource::WaveSource(\"%s\",%p) [%p]",file.c_str(),chan,this);
     if (file == "-") {
 	m_nodata = true;
 	m_brate = 8000;
@@ -141,7 +173,7 @@ WaveSource::WaveSource(const String& file, CallEndpoint* chan, bool autoclose)
 	Debug(DebugWarn,"Opening '%s': error %d: %s",
 	    file.c_str(), errno, ::strerror(errno));
 	m_format.clear();
-	notify();
+	notify(this,"error");
 	return;
     }
     if (file.endsWith(".gsm"))
@@ -162,14 +194,26 @@ WaveSource::WaveSource(const String& file, CallEndpoint* chan, bool autoclose)
 	detectIlbcFormat();
     else if (!file.endsWith(".slin"))
 	Debug(DebugMild,"Unknown format for file '%s', assuming signed linear",file.c_str());
-    if (computeDataRate())
+    if (computeDataRate()) {
+	if (autorepeat)
+	    m_repeatPos = ::lseek(m_fd,0,SEEK_CUR);
+	asyncDelete(s_asyncDelete);
 	start("WaveSource");
+    }
     else {
 	Debug(DebugWarn,"Unable to compute data rate for file '%s'",file.c_str());
-	notify();
+	notify(this,"error");
     }
 }
 
+WaveSource::WaveSource(const char* file, CallEndpoint* chan, bool autoclose)
+    : m_chan(chan), m_fd(-1), m_swap(false), m_brate(0), m_repeatPos(-1),
+      m_total(0), m_time(0), m_autoclose(autoclose), m_autoclean(false),
+      m_nodata(false)
+{
+    Debug(&__plugin,DebugAll,"WaveSource::WaveSource(\"%s\",%p) [%p]",file,chan,this);
+}
+
 WaveSource::~WaveSource()
 {
     Debug(&__plugin,DebugAll,"WaveSource::~WaveSource() [%p] total=%u stamp=%lu",this,m_total,timeStamp());
@@ -189,14 +233,7 @@ WaveSource::~WaveSource()
 
 void WaveSource::detectAuFormat()
 {
-    struct {
-	uint32_t sign;
-	uint32_t offs;
-	uint32_t len;
-	uint32_t form;
-	uint32_t freq;
-	uint32_t chan;
-    } header;
+    AuHeader header;
     if ((::read(m_fd,&header,sizeof(header)) != sizeof(header)) ||
 	(ntohl(header.sign) != 0x2E736E64)) {
 	Debug(DebugMild,"Invalid .au file header, assuming raw signed linear");
@@ -221,12 +258,10 @@ void WaveSource::detectAuFormat()
 	default:
 	    Debug(DebugMild,"Unknown .au format 0x%0X, assuming signed linear",ntohl(header.form));
     }
-#if 0
     if (samp != 8000)
-	m_format = String(samp) + "/" + m_format;
-    if (chan != 1)
+	m_format << "/" << samp;
+    if (chan > 1)
 	m_format = String(chan) + "*" + m_format;
-#endif
 }
 
 void WaveSource::detectWavFormat()
@@ -234,7 +269,6 @@ void WaveSource::detectWavFormat()
     Debug(DebugMild,".wav not supported yet, assuming raw signed linear");
 }
 
-#define ILBC_HEADER_LEN 9
 void WaveSource::detectIlbcFormat()
 {
     char header[ILBC_HEADER_LEN+1];
@@ -276,7 +310,9 @@ void WaveSource::run()
 	if (!alive())
 	    return;
     }
-    m_data.assign(0,(m_brate*20)/1000);
+    unsigned int blen = (m_brate*20)/1000;
+    DDebug(&__plugin,DebugAll,"Consumer found, starting to play data with rate %d [%p]",m_brate,this);
+    m_data.assign(0,blen);
     // start counting time from now
     u_int64_t tpos = Time::now();
     m_time = tpos;
@@ -289,6 +325,17 @@ void WaveSource::run()
 	    }
 	    break;
 	}
+	if (!r) {
+	    if (m_repeatPos >= 0) {
+		DDebug(&__plugin,DebugAll,"Autorepeating from offset %ld [%p]",
+		    m_repeatPos,this);
+		::lseek(m_fd,m_repeatPos,SEEK_SET);
+		m_data.assign(0,blen);
+		r = 1;
+		continue;
+	    }
+	    break;
+	}
 	if (r < (int)m_data.length())
 	    m_data.assign(m_data.data(),r);
 	if (m_swap) {
@@ -303,50 +350,78 @@ void WaveSource::run()
 	    XDebug(&__plugin,DebugAll,"WaveSource sleeping for " FMT64 " usec",dly);
 	    Thread::usleep((unsigned long)dly);
 	}
-	if (!alive())
-	    break;
+	if (!alive()) {
+	    m_autoclose = false;
+	    // if this is a zombie it surely has no owner anymore
+	    m_chan = 0;
+	    notify(0,"replaced");
+	    return;
+	}
 	Forward(m_data,ts);
 	ts += m_data.length()*8000/m_brate;
 	m_total += r;
 	tpos += (r*(u_int64_t)1000000/m_brate);
     } while (r > 0);
-    Debug(&__plugin,DebugAll,"WaveSource [%p] end of data [%p] [%s] ",this,m_chan,m_id.c_str());
+    Debug(&__plugin,DebugAll,"WaveSource '%s' end of data (%u played) chan=%p [%p]",m_id.c_str(),m_total,m_chan,this);
     // at cleanup time deref the data source if we start no disconnector thread
-    m_autoclean = !notify();
+    m_autoclean = !notify(this,"eof");
 }
 
 void WaveSource::cleanup()
 {
-    Debug(&__plugin,DebugAll,"WaveSource [%p] cleanup, total=%u",this,m_total);
+    Debug(&__plugin,DebugAll,"WaveSource cleanup, total=%u, alive=%s, autoclean=%s [%p]",
+	m_total,String::boolText(alive()),String::boolText(m_autoclean),this);
     if (m_autoclean) {
+	asyncDelete(false);
 	if (m_chan && (m_chan->getSource() == this))
 	    m_chan->setSource();
 	else
 	    deref();
+	return;
     }
+    ThreadedSource::cleanup();
 }
 
 void WaveSource::setNotify(const String& id)
 {
     m_id = id;
     if ((m_fd < 0) && !m_nodata)
-	notify();
+	notify(this);
 }
 
-bool WaveSource::notify()
+bool WaveSource::notify(DataSource* source, const char* reason)
 {
+    if (!m_chan) {
+	if (m_id) {
+	    DDebug(&__plugin,DebugAll,"WaveSource enqueueing notify message [%p]",this);
+	    Message* m = new Message("chan.notify");
+	    m->addParam("targetid",m_id);
+	    if (reason)
+		m->addParam("reason",reason);
+	    Engine::enqueue(m);
+	}
+	return false;
+    }
     if (m_id || m_autoclose) {
-	Disconnector *disc = new Disconnector(m_chan,m_id,true,m_autoclose);
+	DDebug(&__plugin,DebugInfo,"Preparing '%s' disconnector for '%s' chan '%s' source=%p [%p]",
+	    reason,m_id.c_str(),(m_chan ? m_chan->id().c_str() : ""),source,this);
+	Disconnector *disc = new Disconnector(m_chan,m_id,source,m_autoclose,reason);
 	return disc->init();
     }
     return false;
 }
 
-WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxlen)
-    : m_chan(chan), m_fd(-1), m_total(0), m_maxlen(maxlen), m_time(0)
+
+WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxlen, const char* format)
+    : m_chan(chan), m_fd(-1), m_swap(false), m_locked(false), m_header(None),
+      m_total(0), m_maxlen(maxlen), m_time(0)
 {
-    Debug(&__plugin,DebugAll,"WaveConsumer::WaveConsumer(\"%s\",%p,%u) [%p]",
-	file.c_str(),chan,maxlen,this);
+    Debug(&__plugin,DebugAll,"WaveConsumer::WaveConsumer(\"%s\",%p,%u,\"%s\") [%p]",
+	file.c_str(),chan,maxlen,format,this);
+    if (format) {
+	m_locked = true;
+	m_format = format;
+    }
     if (file == "-")
 	return;
     else if (file.endsWith(".gsm"))
@@ -359,6 +434,10 @@ WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxl
 	m_format = "ilbc20";
     else if (file.endsWith(".ilbc30"))
 	m_format = "ilbc30";
+    else if (file.endsWith(".lbc"))
+	m_header=Ilbc;
+    else if (file.endsWith(".au"))
+	m_header=Au;
     m_fd = ::open(file.safe(),O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_BINARY,S_IRUSR|S_IWUSR);
     if (m_fd < 0)
 	Debug(DebugWarn,"Creating '%s': error %d: %s",
@@ -381,13 +460,105 @@ WaveConsumer::~WaveConsumer()
     }
 }
 
+void WaveConsumer::writeIlbcHeader() const
+{
+    if (m_format == "ilbc20")
+	::write(m_fd,"#!iLBC20\n",ILBC_HEADER_LEN);
+    else if (m_format == "ilbc30")
+	::write(m_fd,"#!iLBC30\n",ILBC_HEADER_LEN);
+    else
+	Debug(DebugMild,"Invalid iLBC format '%s', not writing header",m_format.c_str());
+}
+
+void WaveConsumer::writeAuHeader()
+{
+    AuHeader header;
+    String fmt = m_format;
+    int chans = 1;
+    int rate = 8000;
+    int sep = fmt.find('*');
+    if (sep > 0)
+	fmt >> chans >> "*";
+    sep = fmt.find('/');
+    if (sep > 0) {
+	rate = fmt.substr(sep+1).toInteger(rate);
+	fmt.assign(fmt,sep);
+    }
+    if (fmt == "slin") {
+	m_swap = true;
+	header.form = ntohl(3);
+    }
+    else if (fmt == "mulaw")
+	header.form = ntohl(1);
+    else if (fmt == "alaw")
+	header.form = ntohl(27);
+    else {
+	Debug(DebugMild,"Invalid au format '%s', not writing header",m_format.c_str());
+	return;
+    }
+    header.sign = htonl(0x2E736E64);
+    header.offs = htonl(sizeof(header));
+    header.freq = ntohl(rate);
+    header.chan = ntohl(chans);
+    header.len = 0;
+    ::write(m_fd,&header,sizeof(header));
+}
+
+bool WaveConsumer::setFormat(const DataFormat& format)
+{
+    if (m_locked || (format == "slin"))
+	return false;
+    bool ok = false;
+    switch (m_header) {
+	case Ilbc:
+	    if ((format == "ilbc20") || (format == "ilbc30"))
+		ok = true;
+	    break;
+	case Au:
+	    if ((format.find("mulaw") >= 0) || (format.find("alaw") >= 0) || (format.find("slin") >= 0))
+		ok = true;
+	    break;
+	default:
+	    break;
+    }
+    if (ok) {
+	DDebug(DebugInfo,"WaveConsumer new format '%s'",format.c_str());
+	m_format = format;
+	m_locked = true;
+	return true;
+    }
+    return false;
+}
+
 void WaveConsumer::Consume(const DataBlock& data, unsigned long tStamp)
 {
     if (!data.null()) {
 	if (!m_time)
 	    m_time = Time::now();
-	if (m_fd >= 0)
-	    ::write(m_fd,data.data(),data.length());
+	if (m_fd >= 0) {
+	    switch (m_header) {
+		case Ilbc:
+		    writeIlbcHeader();
+		    break;
+		case Au:
+		    writeAuHeader();
+		    break;
+		default:
+		    break;
+	    }
+	    m_header = None;
+	    if (m_swap) {
+		unsigned int n = data.length();
+		DataBlock swapped(0,n);
+		const uint16_t* s = (const uint16_t*)data.data();
+		uint16_t* d = (uint16_t*)swapped.data();
+		for (unsigned int i = 0; i < n; i+= 2)
+		    *d++ = ntohs(*s++);
+		::write(m_fd,swapped.data(),swapped.length());
+	    }
+	    else
+		::write(m_fd,data.data(),data.length());
+	}
 	m_total += data.length();
 	if (m_maxlen && (m_total >= m_maxlen)) {
 	    m_maxlen = 0;
@@ -396,7 +567,9 @@ void WaveConsumer::Consume(const DataBlock& data, unsigned long tStamp)
 		m_fd = -1;
 	    }
 	    if (m_chan) {
-		Disconnector *disc = new Disconnector(m_chan,m_id,false,false);
+		DDebug(&__plugin,DebugInfo,"Preparing 'maxlen' disconnector for '%s' chan '%s' in consumer [%p]",
+		    m_id.c_str(),(m_chan ? m_chan->id().c_str() : ""),this);
+		Disconnector *disc = new Disconnector(m_chan,m_id,0,false,"maxlen");
 		m_chan = 0;
 		disc->init();
 	    }
@@ -404,12 +577,17 @@ void WaveConsumer::Consume(const DataBlock& data, unsigned long tStamp)
     }
 }
 
-Disconnector::Disconnector(CallEndpoint* chan, const String& id, bool source, bool disc)
+
+Disconnector::Disconnector(CallEndpoint* chan, const String& id, DataSource* source, bool disc, const char* reason)
     : m_chan(chan), m_msg(0), m_source(source), m_disc(disc)
 {
     if (id) {
-	Message *m = new Message("chan.notify");
+	Message* m = new Message("chan.notify");
+	if (chan)
+	    m->addParam("id",chan->id());
 	m->addParam("targetid",id);
+	if (reason)
+	    m->addParam("reason",reason);
 	m->userData(chan);
 	m_msg = m;
     }
@@ -417,8 +595,10 @@ Disconnector::Disconnector(CallEndpoint* chan, const String& id, bool source, bo
 
 Disconnector::~Disconnector()
 {
-    if (m_msg)
+    if (m_msg) {
+	DDebug(&__plugin,DebugAll,"Disconnector enqueueing notify message [%p]",this);
 	Engine::enqueue(m_msg);
+    }
 }
 
 bool Disconnector::init()
@@ -433,12 +613,16 @@ bool Disconnector::init()
 
 void Disconnector::run()
 {
-    DDebug(&__plugin,DebugAll,"Disconnector::run() chan=%p msg=%p",
-	(void*)m_chan,m_msg);
+    DDebug(&__plugin,DebugAll,"Disconnector::run() chan=%p msg=%p source=%s disc=%s [%p]",
+	(void*)m_chan,m_msg,String::boolText(m_source),String::boolText(m_disc),this);
     if (!m_chan)
 	return;
     if (m_source) {
-	m_chan->setSource();
+	if (m_chan->getSource() == m_source)
+	    m_chan->setSource();
+	else
+	    Debug(&__plugin,DebugMild,"Source %p in channel %p was replaced with %p",
+		m_source,(void*)m_chan,m_chan->getSource());
 	if (m_disc)
 	    m_chan->disconnect("eof");
     }
@@ -450,16 +634,17 @@ void Disconnector::run()
     }
 }
 
-WaveChan::WaveChan(const String& file, bool record, unsigned maxlen)
+
+WaveChan::WaveChan(const String& file, bool record, unsigned maxlen, bool autorepeat, const char* format)
     : Channel(__plugin)
 {
     Debug(this,DebugAll,"WaveChan::WaveChan(%s) [%p]",(record ? "record" : "play"),this);
     if (record) {
-	setConsumer(new WaveConsumer(file,this,maxlen));
+	setConsumer(new WaveConsumer(file,this,maxlen,format));
 	getConsumer()->deref();
     }
     else {
-	setSource(new WaveSource(file,this));
+	setSource(WaveSource::create(file,this,true,autorepeat));
 	getSource()->deref();
     }
 }
@@ -469,9 +654,10 @@ WaveChan::~WaveChan()
     Debug(this,DebugAll,"WaveChan::~WaveChan() %s [%p]",id().c_str(),this);
 }
 
+
 bool AttachHandler::received(Message &msg)
 {
-    int more = 3;
+    int more = 4;
     String src(msg.getValue("source"));
     if (src.null())
 	more--;
@@ -485,11 +671,11 @@ bool AttachHandler::received(Message &msg)
 	    else {
 		Debug(DebugWarn,"Could not attach source with method '%s', use 'play'",
 		    src.matchString(1).c_str());
-		src = "";
+		src.clear();
 	    }
 	}
 	else
-	    src = "";
+	    src.clear();
     }
 
     String cons(msg.getValue("consumer"));
@@ -505,11 +691,11 @@ bool AttachHandler::received(Message &msg)
 	    else {
 		Debug(DebugWarn,"Could not attach consumer with method '%s', use 'record'",
 		    cons.matchString(1).c_str());
-		cons = "";
+		cons.clear();
 	    }
 	}
 	else
-	    cons = "";
+	    cons.clear();
     }
 
     String ovr(msg.getValue("override"));
@@ -525,14 +711,34 @@ bool AttachHandler::received(Message &msg)
 	    else {
 		Debug(DebugWarn,"Could not attach override with method '%s', use 'play'",
 		    ovr.matchString(1).c_str());
-		ovr = "";
+		ovr.clear();
 	    }
 	}
 	else
-	    ovr = "";
+	    ovr.clear();
     }
 
-    if (src.null() && cons.null() && ovr.null())
+    String repl(msg.getValue("replace"));
+    if (repl.null())
+	more--;
+    else {
+	Regexp r("^wave/\\([^/]*\\)/\\(.*\\)$");
+	if (repl.matches(r)) {
+	    if (repl.matchString(1) == "play") {
+		repl = repl.matchString(2);
+		more--;
+	    }
+	    else {
+		Debug(DebugWarn,"Could not attach replacement with method '%s', use 'play'",
+		    repl.matchString(1).c_str());
+		repl.clear();
+	    }
+	}
+	else
+	    repl.clear();
+    }
+
+    if (src.null() && cons.null() && ovr.null() && repl.null())
 	return false;
 
     // if single attach was requested we can return true if everything is ok
@@ -552,7 +758,7 @@ bool AttachHandler::received(Message &msg)
     }
 
     if (!src.null()) {
-	WaveSource* s = new WaveSource(src,ch,false);
+	WaveSource* s = WaveSource::create(src,ch,false,msg.getBoolValue("autorepeat"));
 	ch->setSource(s);
 	s->setNotify(msg.getValue("notify"));
 	s->deref();
@@ -560,7 +766,7 @@ bool AttachHandler::received(Message &msg)
     }
 
     if (!cons.null()) {
-	WaveConsumer* c = new WaveConsumer(cons,ch,maxlen);
+	WaveConsumer* c = new WaveConsumer(cons,ch,maxlen,msg.getValue("format"));
 	c->setNotify(msg.getValue("notify"));
 	ch->setConsumer(c);
 	c->deref();
@@ -574,7 +780,7 @@ bool AttachHandler::received(Message &msg)
 	    ret = false;
 	    break;
 	}
-	WaveSource* s = new WaveSource(ovr,0,false);
+	WaveSource* s = WaveSource::create(ovr,0,false,msg.getBoolValue("autorepeat"));
 	s->setNotify(msg.getValue("notify"));
 	if (DataTranslator::attachChain(s,c,true))
 	    msg.clearParam("override");
@@ -586,10 +792,30 @@ bool AttachHandler::received(Message &msg)
 	break;
     }
 
+    while (!repl.null()) {
+	DataConsumer* c = ch->getConsumer();
+	if (!c) {
+	    Debug(DebugWarn,"Wave replacement '%s' attach request with no consumer!",repl.c_str());
+	    ret = false;
+	    break;
+	}
+	WaveSource* s = WaveSource::create(repl,0,false,msg.getBoolValue("autorepeat"));
+	s->setNotify(msg.getValue("notify"));
+	if (DataTranslator::attachChain(s,c,false))
+	    msg.clearParam("replace");
+	else {
+	    Debug(DebugWarn,"Failed to replacement attach wave '%s' to consumer %p",repl.c_str(),c);
+	    s->deref();
+	    ret = false;
+	}
+	break;
+    }
+
     // Stop dispatching if we handled all requested
     return ret && !more;
 }
 
+
 bool RecordHandler::received(Message &msg)
 {
     int more = 2;
@@ -654,14 +880,14 @@ bool RecordHandler::received(Message &msg)
     }
 
     if (!c1.null()) {
-	WaveConsumer* c = new WaveConsumer(c1,ch,maxlen);
+	WaveConsumer* c = new WaveConsumer(c1,ch,maxlen,msg.getValue("format"));
 	c->setNotify(msg.getValue("notify"));
 	de->setCallRecord(c);
 	c->deref();
     }
 
     if (!c2.null()) {
-	WaveConsumer* c = new WaveConsumer(c2,ch,maxlen);
+	WaveConsumer* c = new WaveConsumer(c2,ch,maxlen,msg.getValue("format"));
 	c->setNotify(msg.getValue("notify"));
 	de->setPeerRecord(c);
 	c->deref();
@@ -671,6 +897,7 @@ bool RecordHandler::received(Message &msg)
     return !more;
 }
 
+
 bool WaveFileDriver::msgExecute(Message& msg, String& dest)
 {
     Regexp r("^\\([^/]*\\)/\\(.*\\)$");
@@ -692,7 +919,7 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
     if (ch) {
 	Debug(this,DebugInfo,"%s wave file '%s'", (meth ? "Record to" : "Play from"),
 	    dest.matchString(2).c_str());
-	WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen);
+	WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen,msg.getBoolValue("autorepeat"),msg.getValue("format"));
 	if (ch->connect(c,msg.getValue("reason"))) {
 	    msg.setParam("peerid",c->id());
 	    c->deref();
@@ -726,7 +953,7 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
     }
     m = "call.execute";
     m.addParam("callto",callto);
-    WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen);
+    WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen,msg.getBoolValue("autorepeat"),msg.getValue("format"));
     m.setParam("id",c->id());
     m.userData(c);
     if (Engine::dispatch(m)) {
@@ -749,6 +976,7 @@ void WaveFileDriver::initialize()
 {
     Output("Initializing module WaveFile");
     setup();
+    s_asyncDelete = Engine::config().getBoolValue("hacks","asyncdelete",true);
     if (!m_handler) {
 	m_handler = new AttachHandler;
 	Engine::install(m_handler);
