@@ -37,6 +37,21 @@
 using namespace TelEngine;
 namespace { // anonymous
 
+TokenDict dict_payloads[] = {
+    {"gsm",    IAXFormat::GSM},
+    {"ilbc30", IAXFormat::ILBC},
+    {"speex",  IAXFormat::SPEEX},
+    {"lpc10",  IAXFormat::LPC10},
+    {"mulaw",  IAXFormat::ULAW},
+    {"alaw",   IAXFormat::ALAW},
+    {"g723",   IAXFormat::G723_1},
+    {"g729",   IAXFormat::G729A},
+    {"adpcm",  IAXFormat::ADPCM},
+    {"mp3",    IAXFormat::MP3},
+    {"slin",   IAXFormat::SLIN},
+    {0, 0}
+};
+
 static TokenDict dict_tos[] = {
     { "lowdelay", Socket::LowDelay },
     { "throughput", Socket::MaxThroughput },
@@ -141,6 +156,13 @@ public:
      */
     void regTerminate(IAXEvent* event);
 
+    /*
+     * Process an authentication request for a Register/Unregister operation
+     * This method is thread safe
+     * @param event The event (AuthReq)
+     */
+    void regAuthReq(IAXEvent* event);
+
     /*
      * Timer notification
      * This method is thread safe
@@ -230,10 +252,11 @@ public:
      * @param transTimeout Timeout (in seconds) on remote request of transactions belonging to this engine
      * @param maxFullFrameDataLen Max full frame IE list (buffer) length
      * @param trunkSendInterval Send trunk meta frame interval
+     * @param authRequired Automatically challenge all clients for authentication
      */
     YIAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
 	u_int16_t authTimeout, u_int16_t transTimeout,
-	u_int16_t maxFullFrameDataLen, u_int32_t trunkSendInterval);
+	u_int16_t maxFullFrameDataLen, u_int32_t trunkSendInterval, bool authRequired);
 
     virtual ~YIAXEngine()
 	{}
@@ -286,8 +309,11 @@ protected:
 
     /*
      * Event handler for incoming registration transactions.
+     * @param event The event.
+     * @param first True if this is the first request.
+     *  False if it is a response to an authentication request.
      */
-    void processRemoteReg(IAXEvent* event);
+    void processRemoteReg(IAXEvent* event,bool first);
 
     /*
      * Send Register/Unregister messages to Engine
@@ -339,6 +365,24 @@ public:
     inline YIAXEngine* getEngine() const
 	{ return m_iaxEngine; }
 
+    // Update codecs from 'formats' parameter of a message
+    // @param codecs Codec list to update
+    // @param formats The 'formats' parameter of a message
+    // @return False if formtas is not 0 and the result is 0 (no intersection)
+    bool updateCodecsFromRoute(u_int32_t& codecs, const char* formats);
+
+    // Create a format list from codecs
+    void createFormatList(String& dest, u_int32_t codecs);
+
+    // Dispatch user.auth
+    // @tr The IAX transaction
+    // @param response True if it is a response.
+    // @param requestAuth True on exit: the caller should request authentication
+    // @param invalidAuth True on exit: authentication response is incorrect
+    // @return False if not authenticated
+    bool userAuth(IAXTransaction* tr, bool response, bool& requestAuth,
+	bool& invalidAuth);
+
 protected:
     YIAXEngine* m_iaxEngine;
     u_int32_t m_defaultCodec;
@@ -418,6 +462,7 @@ protected:
     void startAudioIn();
     void startAudioOut();
     void evAuthRep(IAXEvent* event);
+    void evAuthReq(IAXEvent* event);
     // Safe deref the connection if the reference counter was increased during registration
     void safeDeref();
     bool safeRefIncrease();
@@ -573,6 +618,28 @@ void YIAXLineContainer::regTerminate(IAXEvent* event)
     line->m_state = YIAXLine::Idle;
 }
 
+// Handle registration related authentication
+void YIAXLineContainer::regAuthReq(IAXEvent* event)
+{
+    Lock lock(this);
+    if (!event || event->type() != IAXEvent::AuthReq)
+	return;
+    IAXTransaction* trans = event->getTransaction();
+    if (!trans)
+	return;
+    YIAXLine* line = findLine(static_cast<YIAXLine*>(trans->getUserData()));
+    if (!line) {
+	Debug(&iplugin,DebugAll,"Lines: NO LINE");
+	return;
+    }
+    // Send auth reply
+    String response;
+    if (!iplugin.getEngine())
+	return;
+    iplugin.getEngine()->getMD5FromChallenge(response,trans->challenge(),line->password());
+    trans->sendAuthReply(response);
+}
+
 // Handle registration related events
 void YIAXLineContainer::handleEvent(IAXEvent* event)
 {
@@ -582,6 +649,9 @@ void YIAXLineContainer::handleEvent(IAXEvent* event)
 	case IAXEvent::Timeout:
 	    regTerminate(event);
 	    break;
+	case IAXEvent::AuthReq:
+	    regAuthReq(event);
+	    break;
 	default:
 	    iplugin.getEngine()->defaultEventHandler(event);
     }
@@ -757,7 +827,7 @@ void YIAXLineContainer::clear(bool forced)
 // Run the socket listening thread
 void YIAXListener::run()
 {
-    Debug(m_engine,DebugAll,"%s started",currentName());
+    DDebug(m_engine,DebugAll,"%s started",currentName());
     SocketAddr addr;
     m_engine->readSocket(addr);
 }
@@ -765,24 +835,27 @@ void YIAXListener::run()
 // Run the event retriving thread
 void YIAXGetEvent::run()
 {
-    Debug(m_engine,DebugAll,"%s started",currentName());
+    DDebug(m_engine,DebugAll,"%s started",currentName());
     m_engine->runGetEvents();
 }
 
 // Run the trunk sending thread
 void YIAXTrunking::run()
 {
-    Debug(m_engine,DebugAll,"%s started",currentName());
+    DDebug(m_engine,DebugAll,"%s started",currentName());
     m_engine->runProcessTrunkFrames();
 }
 
 /**
  * YIAXEngine
  */
-YIAXEngine::YIAXEngine(const char* iface, int port, u_int16_t transListCount, u_int16_t retransCount, u_int16_t retransInterval,
-	u_int16_t authTimeout, u_int16_t transTimeout, u_int16_t maxFullFrameDataLen, u_int32_t trunkSendInterval)
-    : IAXEngine(iface,port,transListCount,retransCount,retransInterval,authTimeout,transTimeout,
-      maxFullFrameDataLen,iplugin.defaultCodec(),iplugin.codecs(),trunkSendInterval),
+YIAXEngine::YIAXEngine(const char* iface, int port, u_int16_t transListCount,
+	u_int16_t retransCount, u_int16_t retransInterval, u_int16_t authTimeout,
+	u_int16_t transTimeout, u_int16_t maxFullFrameDataLen,
+	u_int32_t trunkSendInterval, bool authRequired)
+    : IAXEngine(iface,port,transListCount,retransCount,retransInterval,authTimeout,
+	transTimeout,maxFullFrameDataLen,iplugin.defaultCodec(),iplugin.codecs(),
+	trunkSendInterval,authRequired),
       m_threadsCreated(false)
 {
 }
@@ -793,7 +866,8 @@ void YIAXEngine::processMedia(IAXTransaction* transaction, DataBlock& data, u_in
     if (transaction)
 	if (transaction->getUserData())
 	    if ((static_cast<YIAXConnection*>(transaction->getUserData()))->getSource())
-		(static_cast<YIAXSource*>((static_cast<YIAXConnection*>(transaction->getUserData()))->getSource()))->Forward(data,tStamp);
+		// Multiply tStamp with 8 to keep the consumer satisfied
+		(static_cast<YIAXSource*>((static_cast<YIAXConnection*>(transaction->getUserData()))->getSource()))->Forward(data,tStamp * 8);
 	    else {
 		XDebug(this,DebugAll,"YIAXEngine - processMedia. No media source");
 	    }
@@ -841,8 +915,26 @@ IAXTransaction* YIAXEngine::call(SocketAddr& addr, NamedList& params)
     ieList.appendString(IAXInfoElement::CALLING_NAME,params.getValue("callername"));
     ieList.appendString(IAXInfoElement::CALLED_NUMBER,params.getValue("called"));
     ieList.appendString(IAXInfoElement::CALLED_CONTEXT,params.getValue("iaxcontext"));
-    ieList.appendNumeric(IAXInfoElement::FORMAT,iplugin.defaultCodec(),4);
-    ieList.appendNumeric(IAXInfoElement::CAPABILITY,iplugin.codecs(),4);
+    // Set format and capabilities
+    u_int32_t codecs = iplugin.codecs();
+    if (!iplugin.updateCodecsFromRoute(codecs,params.getValue("formats"))) {
+	DDebug(this,DebugAll,"Outgoing call failed: No codecs.");
+	params.setParam("error","nomedia");
+	return 0;
+    }
+    u_int32_t format = iplugin.defaultCodec();
+    if (!(format & codecs)) {
+	for (format = 1; format < 0x10000; format = format << 1)
+	    if (format & codecs)
+		break;
+	if (format >= 0x10000) {
+	    DDebug(this,DebugAll,"Outgoing call failed: No preffered format.");
+	    params.setParam("error","nomedia");
+	    return 0;
+	}
+    }
+    ieList.appendNumeric(IAXInfoElement::FORMAT,format,4);
+    ieList.appendNumeric(IAXInfoElement::CAPABILITY,codecs,4);
     return startLocalTransaction(IAXTransaction::New,addr,ieList);
 }
 
@@ -901,8 +993,8 @@ void YIAXEngine::processEvent(IAXEvent* event)
 	    break;
 	case IAXTransaction::RegReq:
 	case IAXTransaction::RegRel:
-	    if (event->type() == IAXEvent::New)
-		processRemoteReg(event);
+	    if (event->type() == IAXEvent::New || event->type() == IAXEvent::AuthRep)
+		processRemoteReg(event,(event->type() == IAXEvent::New));
 	    else
 		if (event->getTransaction()->getUserData())
 		    s_lines.handleEvent(event);
@@ -913,38 +1005,44 @@ void YIAXEngine::processEvent(IAXEvent* event)
 }
 
 // Process events for remote users registering to us
-void YIAXEngine::processRemoteReg(IAXEvent* event)
+void YIAXEngine::processRemoteReg(IAXEvent* event, bool first)
 {
     IAXTransaction* tr = event->getTransaction();
-
     Debug(this,DebugAll,"processRemoteReg: %s username: '%s'",
 	tr->type() == IAXTransaction::RegReq?"Register":"Unregister",tr->username().c_str());
-    Message msg("user.auth");
-    msg.addParam("username",tr->username());
-    msg.addParam("ip_host",tr->remoteAddr().host());
-    msg.addParam("ip_port",String(tr->remoteAddr().port()));
-    if (!Engine::dispatch(msg)) {
-	// Not authenticated
-	Debug(this,DebugAll,"processRemoteReg. Not authenticated. Reject");
-	tr->sendReject();
+    // Check for automatomatically authentication request if it's the first request
+    if (first && iplugin.getEngine()->authRequired()) {
+	Debug(this,DebugAll,"processRemoteReg. Request authentication");
+	tr->sendAuth();
 	return;
     }
-    String password = msg.retValue();
-    if (password.null()) {
-	// Authenticated, no password. Try to (un)register
+    // Authenticated: register/unregister
+    bool requestAuth = false, invalidAuth = false;
+    if (iplugin.userAuth(tr,!first,requestAuth,invalidAuth)) {
+	// Authenticated. Try to (un)register
 	if (userreg(tr,event->subclass() == IAXControl::RegRel)) {
 	    Debug(this,DebugAll,"processRemoteReg. Authenticated and (un)registered. Ack");
 	    tr->sendAccept();
 	}
 	else {
 	    Debug(this,DebugAll,"processRemoteReg. Authenticated but not (un)registered. Reject");
-	    tr->sendReject();
+	    tr->sendReject("not registered");
 	}
 	return;
     }
-    // Authenticated, password required
-    Debug(this,DebugAll,"processRemoteReg. Request authentication");
-    tr->sendAuth(password);
+    // First request: check if we should request auth
+    const char* reason = 0;
+    if (first && requestAuth) {
+	Debug(this,DebugAll,"processRemoteReg. Request authentication");
+	tr->sendAuth();
+	return;
+    }
+    else if (invalidAuth)
+	 reason = IAXTransaction::s_iax_modInvalidAuth;
+    if (!reason)
+	reason = "not authenticated";
+    Debug(this,DebugAll,"processRemoteReg. Not authenticated ('%s'). Reject",reason);
+    tr->sendReject(reason);
 }
 
 // Build and dispatch the user.(un)register message
@@ -1011,13 +1109,15 @@ void YIAXDriver::initialize()
     u_int32_t fallback = 0;
     String preferred = s_cfg.getValue("formats","preferred");
     bool def = s_cfg.getBoolValue("formats","default",true);
-    for (int i = 0; IAXFormat::audioData[i].token; i++) {
-	if (s_cfg.getBoolValue("formats",IAXFormat::audioData[i].token,
-	    def && DataTranslator::canConvert(IAXFormat::audioData[i].token))) {
-	    m_codecs |= IAXFormat::audioData[i].value;
-	    fallback = IAXFormat::audioData[i].value;
+    for (int i = 0; dict_payloads[i].token; i++) {
+	if (s_cfg.getBoolValue("formats",dict_payloads[i].token,
+	    def && DataTranslator::canConvert(dict_payloads[i].token))) {
+	    XDebug(this,DebugAll,"Adding supported codec %u: '%s'.",
+		dict_payloads[i].value,dict_payloads[i].token);
+	    m_codecs |= dict_payloads[i].value;
+	    fallback = dict_payloads[i].value;
 	    // Set default (desired) codec
-	    if (preferred == IAXFormat::audioData[i].token)
+	    if (preferred == dict_payloads[i].token)
 		m_defaultCodec = fallback;
 	}
     }
@@ -1029,6 +1129,7 @@ void YIAXDriver::initialize()
     // Port and interface
     m_port = s_cfg.getIntValue("general","port",4569);
     String iface = s_cfg.getValue("general","addr");
+    bool authReq = s_cfg.getBoolValue("registrar","auth_required",true);
     unlock();
     setup();
     // We need channels to be dropped on shutdown
@@ -1045,7 +1146,7 @@ void YIAXDriver::initialize()
     if (!m_iaxEngine) {
 	Engine::install(new YIAXRegDataHandler);
 	m_iaxEngine = new YIAXEngine(iface,m_port,transListCount,retransCount,retransInterval,authTimeout,
-		transTimeout,maxFullFrameDataLen,trunkSendInterval);
+		transTimeout,maxFullFrameDataLen,trunkSendInterval,authReq);
 	m_iaxEngine->debugChain(this);
 	int tos = s_cfg.getIntValue("general","tos",dict_tos,0);
 	if (tos) {
@@ -1053,9 +1154,15 @@ void YIAXDriver::initialize()
 		Debug(this,DebugWarn,"Could not set IP TOS to 0x%02x",tos);
 	}
     }
-    int readThreadCount = 3;
-    int eventThreadCount = 3;
-    int trunkingThreadCount = 1;
+    int readThreadCount = s_cfg.getIntValue("general","read_threads",Engine::clientMode() ? 1 : 3);
+    if (readThreadCount < 1)
+	readThreadCount = 1;
+    int eventThreadCount = s_cfg.getIntValue("general","event_threads",Engine::clientMode() ? 1 : 3);
+    if (eventThreadCount < 1)
+	eventThreadCount = 1;
+    int trunkingThreadCount = s_cfg.getIntValue("general","trunk_threads",1);
+    if (trunkingThreadCount < 1)
+	trunkingThreadCount = 1;
     m_iaxEngine->start(readThreadCount,eventThreadCount,trunkingThreadCount);
 }
 
@@ -1144,6 +1251,85 @@ bool YIAXDriver::received(Message& msg, int id)
     return Driver::received(msg,id);
 }
 
+bool YIAXDriver::updateCodecsFromRoute(u_int32_t& codecs, const char* formats)
+{
+// Extract individual codecs from 'formats'
+// Check if IAXFormat contains it
+// Before exiting: update 'codecs'
+    if (!formats)
+	return true;
+    u_int32_t codec_formats = 0;
+    for (u_int32_t i = 0; formats[i];) {
+	// Skip separator(s)
+	for (; formats[i] && formats[i] == ','; i++) ;
+	// Find first separator
+	u_int32_t start = i;
+	for (; formats[i] && formats[i] != ','; i++) ;
+	// Get format
+	if (start != i) {
+	    // Get string
+	    String tmp(formats + start,i - start);
+	    // Get format from IAXFormat::audioData
+	    u_int32_t format = 0;
+	    for (u_int32_t j = 0; dict_payloads[j].value; j++)
+		if (tmp == dict_payloads[j].token) {
+		    format = dict_payloads[j].value;
+		    break;
+		}
+	    if (format)
+		codec_formats |= format;
+	}
+    }
+    // Set intersection
+    codecs &= codec_formats;
+    return codecs != 0;
+}
+
+void YIAXDriver::createFormatList(String& dest, u_int32_t codecs)
+{
+    for (u_int32_t i = 0; dict_payloads[i].token; i++) {
+	if (!(codecs & dict_payloads[i].value))
+	    continue;
+	dest.append(dict_payloads[i].token,",");
+    }
+}
+
+bool YIAXDriver::userAuth(IAXTransaction* tr, bool response, bool& requestAuth,
+	bool& invalidAuth)
+{
+    requestAuth = invalidAuth = false;
+    // Create and dispatch user.auth
+    Message msg("user.auth");
+    msg.addParam("protocol","iax");
+    msg.addParam("username",tr->username());
+    msg.addParam("called",tr->calledNo());
+    msg.addParam("caller",tr->callingNo());
+    msg.addParam("callername",tr->callingName());
+    msg.addParam("ip_host",tr->remoteAddr().host());
+    msg.addParam("ip_port",String(tr->remoteAddr().port()));
+    if (response) {
+	msg.addParam("nonce",tr->challenge());
+	msg.addParam("response",tr->authdata());
+    }
+    if (!Engine::dispatch(msg))
+	return false;
+    String pwd = msg.retValue();
+    // We have a password
+    if (pwd) {
+	// Not a response: request authentication
+	if (!response) {
+	    requestAuth = true;
+	    return false;
+	}
+	// Check response
+	if (!IAXEngine::isMD5ChallengeCorrect(tr->authdata(),tr->challenge(),pwd)) {
+	    invalidAuth = true;
+	    return false;
+	}
+    }
+    return true;
+}
+
 /**
  * IAXConsumer
  */
@@ -1195,7 +1381,7 @@ YIAXConnection::YIAXConnection(YIAXEngine* iaxEngine, IAXTransaction* transactio
 {
     DDebug(this,DebugAll,"YIAXConnection::YIAXConnection [%p]",this);
     setMaxcall(msg);
-    Message* m = message("chan.startup");
+    Message* m = message("chan.startup",msg);
     m->setParam("direction",status());
     if (transaction)
 	m_address << transaction->remoteAddr().host() << ":" << transaction->remoteAddr().port();
@@ -1244,14 +1430,13 @@ void YIAXConnection::callRejected(const char* error, const char* reason, const M
 	reason = error;
     DDebug(this,DebugInfo,"callRejected [%p]. Error: '%s'",this,error);
     String s(error);
-    m_mutexTrans.lock();
+    Lock lock(m_mutexTrans);
     if (m_transaction && (s == "noauth") && safeRefIncrease()) {
-	Debug(this,DebugAll,"callRejected [%p]. Requesting authentication",this);
-	m_transaction->sendAuth(m_password);
-	m_mutexTrans.unlock();
+	Debug(this,DebugAll,"callRejected [%p]. Request authentication",this);
+	m_transaction->sendAuth();
 	return;
     }
-    m_mutexTrans.unlock();
+    lock.drop();
     hangup(reason,true);
 }
 
@@ -1390,7 +1575,8 @@ void YIAXConnection::handleEvent(IAXEvent* event)
 	    break;
 	case IAXEvent::Timeout:
 	    DDebug(this,DebugNote,"YIAXConnection - TIMEOUT. Transaction: %u,%u, Frame: %u,%u ",
-		event->getTransaction()->localCallNo(),event->getTransaction()->remoteCallNo(),event->frameType(),event->subclass());
+		event->getTransaction()->localCallNo(),event->getTransaction()->remoteCallNo(),
+		event->frameType(),event->subclass());
 	    m_reason = "Timeout";
 	    break;
 	case IAXEvent::Busy:
@@ -1400,6 +1586,9 @@ void YIAXConnection::handleEvent(IAXEvent* event)
 	case IAXEvent::AuthRep:
 	    evAuthRep(event);
 	    break;
+	case IAXEvent::AuthReq:
+	    evAuthReq(event);
+	    break;
 	default:
 	    iplugin.getEngine()->defaultEventHandler(event);
 	    if (!m_transaction)
@@ -1455,11 +1644,16 @@ bool YIAXConnection::route(bool authenticated)
 	    hangup(IAXTransaction::s_iax_modNoMediaFormat,true);
 	    return false;
 	}
-	// advertise the not yet authenticated username
+	// Advertise the not yet authenticated username
 	if (m_transaction->username())
 	    m->addParam("authname",m_transaction->username());
+	// Set 'formats' parameter
+	String formats;
+	iplugin.createFormatList(formats,m_transaction->capability());
+	m->addParam("formats",formats);
     }
     m->addParam("called",m_transaction->calledNo());
+    m->addParam("caller",m_transaction->callingNo());
     m->addParam("callername",m_transaction->callingName());
     m->addParam("ip_host",m_transaction->remoteAddr().host());
     m->addParam("ip_port",String(m_transaction->remoteAddr().port()));
@@ -1476,7 +1670,7 @@ void YIAXConnection::startAudioIn()
     if (m_transaction)
 	format = m_transaction->formatIn();
     m_mutexTrans.unlock();
-    const char* formatText = IAXFormat::audioText(format);
+    const char* formatText = lookup(format,dict_payloads);
     setSource(new YIAXSource(this,format,formatText));
     getSource()->deref();
     DDebug(this,DebugAll,"startAudioIn. Format %u: '%s'",format,formatText);
@@ -1492,7 +1686,7 @@ void YIAXConnection::startAudioOut()
     if (m_transaction)
 	format = m_transaction->formatOut();
     m_mutexTrans.unlock();
-    const char* formatText = (char*)IAXFormat::audioText(format);
+    const char* formatText = lookup(format,dict_payloads);
     setConsumer(new YIAXConsumer(this,format,formatText));
     getConsumer()->deref();
     DDebug(this,DebugAll,"startAudioOut. Format %u: '%s'",format,formatText);
@@ -1501,29 +1695,25 @@ void YIAXConnection::startAudioOut()
 void YIAXConnection::evAuthRep(IAXEvent* event)
 {
     DDebug(this,DebugAll,"YIAXConnection - AUTHREP");
-    IAXTransaction* tr = event->getTransaction();
-    // Try to obtain a password from Engine
-    Message msg("user.auth");
-    msg.addParam("username",tr->username());
-    if (!Engine::dispatch(msg)) {
-	// NOT Authenticated
-	hangup(event,"",true);
+    bool requestAuth, invalidAuth;
+    if (iplugin.userAuth(event->getTransaction(),true,requestAuth,invalidAuth)) {
+	// Authenticated. Route the user.
+	route(true);
 	return;
     }
-    String pwd = msg.retValue();
-    if (pwd.null()) {
-	// Authenticated
-	tr->sendAccept();
-	return;
+    const char* reason = IAXTransaction::s_iax_modInvalidAuth.c_str();
+    DDebug(this,DebugAll,"Not authenticated. Reason: '%s'. Reject.",reason);
+    hangup(event,reason,true);
+}
+
+void YIAXConnection::evAuthReq(IAXEvent* event)
+{
+    DDebug(this,DebugAll,"YIAXConnection - AUTHREQ");
+    String response;
+    if (m_iaxEngine && m_transaction) {
+	m_iaxEngine->getMD5FromChallenge(response,m_transaction->challenge(),m_password);
+	m_transaction->sendAuthReply(response);
     }
-    if (!IAXEngine::isMD5ChallengeCorrect(tr->authdata(),tr->challenge(),pwd)) {
-	// Incorrect data received
-	DDebug(this,DebugAll,"AUTHREP - Incorrect MD5 answer. Reject.");
-	hangup(event,IAXTransaction::s_iax_modInvalidAuth,true);
-	return;
-    }
-    // Password is correct. Route the user.
-    route(true);
 }
 
 // Get rid of the extra reference
@@ -1624,7 +1814,7 @@ bool IAXURI::setAddr(SocketAddr& dest)
     if (!m_host.length())
 	return false;
     dest.host(m_host);
-    dest.port(m_port ? m_port : iplugin.port());
+    dest.port(m_port ? m_port : 0);
     return true;
 }
 
