@@ -78,7 +78,11 @@ static int s_maxport = MAX_PORT;
 static int s_bufsize = BUF_SIZE;
 static String s_tos;
 static bool s_autoaddr = true;
+static bool s_anyssrc = false;
+static bool s_rtcp = true;
+static bool s_drill = false;
 
+static int s_sleep = 5;
 static int s_minjitter = 0;
 static int s_maxjitter = 0;
 
@@ -94,8 +98,10 @@ class YRTPWrapper : public RefObject
 public:
     YRTPWrapper(const char *localip, CallEndpoint* conn = 0, const char* media = "audio", RTPSession::Direction direction = RTPSession::SendRecv, bool rtcp = true);
     ~YRTPWrapper();
+    virtual void* getObject(const String &name) const;
     void setupRTP(const char* localip, bool rtcp);
     bool startRTP(const char* raddr, unsigned int rport, const Message& msg);
+    bool setRemote(const char* raddr, unsigned int rport, const Message& msg);
     bool sendDTMF(char dtmf, int duration = 0);
     void gotDTMF(char tone);
     inline YRTPSession* rtp() const
@@ -138,9 +144,10 @@ private:
 
 class YRTPSession : public RTPSession
 {
+    friend class RTPSession;
 public:
     inline YRTPSession(YRTPWrapper* wrap)
-	: m_wrap(wrap), m_resync(false)
+	: m_wrap(wrap), m_resync(false), m_anyssrc(false)
 	{ }
     virtual ~YRTPSession();
     virtual bool rtpRecvData(bool marker, unsigned int timestamp,
@@ -151,9 +158,12 @@ public:
     virtual void rtpNewSSRC(u_int32_t newSsrc);
     inline void resync()
 	{ m_resync = true; }
+    inline void anySSRC(bool acceptAny = true)
+	{ m_anyssrc = acceptAny; }
 private:
     YRTPWrapper* m_wrap;
     bool m_resync;
+    bool m_anyssrc;
 };
 
 class YRTPSource : public DataSource
@@ -211,6 +221,7 @@ public:
     virtual ~YRTPPlugin();
     virtual void initialize();
     virtual void statusParams(String& str);
+    virtual void statusDetail(String& str);
 private:
     bool m_first;
 };
@@ -263,6 +274,13 @@ YRTPWrapper::~YRTPWrapper()
     s_mutex.unlock();
 }
 
+void* YRTPWrapper::getObject(const String &name) const
+{
+    if (name == "Socket")
+	return m_rtp ? m_rtp->rtpSock() : 0;
+    return RefObject::getObject(name);
+}
+
 YRTPWrapper* YRTPWrapper::find(const CallEndpoint* conn, const String& media)
 {
     Lock lock(s_mutex);
@@ -321,6 +339,16 @@ void YRTPWrapper::setupRTP(const char* localip, bool rtcp)
     Debug(&splugin,DebugWarn,"YRTPWrapper [%p] RTP bind failed in range %d-%d",this,minport,maxport);
 }
 
+bool YRTPWrapper::setRemote(const char* raddr, unsigned int rport, const Message& msg)
+{
+    SocketAddr addr(AF_INET);
+    if (!(addr.host(raddr) && addr.port(rport) && m_rtp->remoteAddr(addr,msg.getBoolValue("autoaddr",s_autoaddr)))) {
+	Debug(&splugin,DebugWarn,"RTP failed to set remote address %s:%d [%p]",raddr,rport,this);
+	return false;
+    }
+    return true;
+}
+
 bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message& msg)
 {
     Debug(&splugin,DebugAll,"YRTPWrapper::startRTP(\"%s\",%u) [%p]",raddr,rport,this);
@@ -331,6 +359,7 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
 
     if (m_bufsize) {
 	DDebug(&splugin,DebugAll,"Wrapper attempted to restart RTP! [%p]",this);
+	setRemote(raddr,rport,msg);
 	m_rtp->resync();
 	return true;
     }
@@ -343,6 +372,7 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
     const char* format = msg.getValue("format");
     p = msg.getValue("tos",s_tos);
     int tos = p.toInteger(dict_tos,0);
+    int msec = msg.getIntValue("msleep",s_sleep);
 
     if (!format)
 	format = lookup(payload, dict_payloads);
@@ -370,12 +400,9 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
     int minJitter = msg.getIntValue("minjitter",s_minjitter);
     int maxJitter = msg.getIntValue("maxjitter",s_maxjitter);
 
-    bool autoaddr = msg.getBoolValue("autoaddr",s_autoaddr);
-    SocketAddr addr(AF_INET);
-    if (!(addr.host(raddr) && addr.port(rport) && m_rtp->remoteAddr(addr,autoaddr))) {
-	Debug(&splugin,DebugWarn,"RTP failed to set remote address %s:%d [%p]",raddr,rport,this);
+    if (!setRemote(raddr,rport,msg))
 	return false;
-    }
+    m_rtp->anySSRC(msg.getBoolValue("anyssrc",s_anyssrc));
     // Change format of source and/or consumer,
     //  reinstall them to rebuild codec chains
     if (m_source) {
@@ -401,11 +428,13 @@ bool YRTPWrapper::startRTP(const char* raddr, unsigned int rport, const Message&
 	    m_consumer->deref();
 	}
     }
-    if (!(m_rtp->initGroup() && m_rtp->direction(m_dir)))
+    if (!(m_rtp->initGroup(msec) && m_rtp->direction(m_dir)))
 	return false;
     m_rtp->dataPayload(payload);
     m_rtp->eventPayload(evpayload);
     m_rtp->setTOS(tos);
+    if (msg.getBoolValue("drillhole",s_drill))
+	m_rtp->drillHole();
 //    if (maxJitter > 0)
 //	m_rtp->setDejitter(minJitter*1000,maxJitter*1000);
     m_bufsize = s_bufsize;
@@ -507,7 +536,7 @@ void YRTPSession::rtpNewPayload(int payload, unsigned int timestamp)
 
 void YRTPSession::rtpNewSSRC(u_int32_t newSsrc)
 {
-    if (m_resync && receiver()) {
+    if ((m_anyssrc || m_resync) && receiver()) {
 	m_resync = false;
 	Debug(&splugin,DebugInfo,"Changing SSRC from %08X to %08X in wrapper %p",
 	    receiver()->ssrc(),newSsrc,m_wrap);
@@ -635,7 +664,7 @@ bool AttachHandler::received(Message &msg)
     if (!w)
 	w = YRTPWrapper::find(msg.getValue("rtpid"));
     if (!w) {
-	w = new YRTPWrapper(lip,ch,media);
+	w = new YRTPWrapper(lip,ch,media,RTPSession::SendRecv,msg.getBoolValue("rtcp",s_rtcp));
 	w->setMaster(msg.getValue("id"));
 
 	if (!src.null()) {
@@ -687,31 +716,30 @@ bool RtpHandler::received(Message &msg)
 	direction = RTPSession::SendOnly;
     }
 
-    if (!(d_recv || d_send))
-	return false;
-
+    YRTPWrapper* w = 0;
     const char* media = msg.getValue("media","audio");
     CallEndpoint *ch = static_cast<CallEndpoint*>(msg.userData());
-    if (!ch) {
-	if (d_recv)
-	    Debug(&splugin,DebugWarn,"RTP recv request with no call channel!");
-	if (d_send)
-	    Debug(&splugin,DebugWarn,"RTP send request with no call channel!");
-	return false;
+    if (ch) {
+	w = YRTPWrapper::find(ch,media);
+	if (w)
+	    Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint",w);
     }
-
-    String rip(msg.getValue("remoteip"));
-    String rport(msg.getValue("remoteport"));
-
-    YRTPWrapper *w = YRTPWrapper::find(ch,media);
-    if (w)
-	Debug(&splugin,DebugAll,"Wrapper %p found by CallEndpoint",w);
     if (!w) {
 	w = YRTPWrapper::find(msg.getValue("rtpid"));
 	if (w)
 	    Debug(&splugin,DebugAll,"Wrapper %p found by ID",w);
     }
+    if (!(ch || w)) {
+	Debug(&splugin,DebugWarn,"Neither call channel nor RTP wrapper found!");
+	return false;
+    }
+
+    String rip(msg.getValue("remoteip"));
+
     if (!w) {
+	// it would be pointless to create an unreferenced wrapper
+	if (!(d_recv || d_send))
+	    return false;
 	String lip(msg.getValue("localip"));
 	if (lip.null())
 	    YRTPWrapper::guessLocal(rip,lip);
@@ -721,7 +749,7 @@ bool RtpHandler::received(Message &msg)
 	}
 	msg.setParam("localip",lip);
 
-	w = new YRTPWrapper(lip,ch,media,direction,msg.getBoolValue("rtcp",true));
+	w = new YRTPWrapper(lip,ch,media,direction,msg.getBoolValue("rtcp",s_rtcp));
 	w->setMaster(msg.getValue("id"));
     }
     else {
@@ -729,13 +757,13 @@ bool RtpHandler::received(Message &msg)
 	w->addDirection(direction);
     }
 
-    if (d_recv && !ch->getSource(media)) {
+    if (d_recv && ch && !ch->getSource(media)) {
 	YRTPSource* s = new YRTPSource(w);
 	ch->setSource(s,media);
 	s->deref();
     }
 
-    if (d_send && !ch->getConsumer(media)) {
+    if (d_send && ch && !ch->getConsumer(media)) {
 	YRTPConsumer* c = new YRTPConsumer(w);
 	ch->setConsumer(c,media);
 	c->deref();
@@ -744,11 +772,14 @@ bool RtpHandler::received(Message &msg)
     if (w->deref())
 	return false;
 
+    String rport(msg.getValue("remoteport"));
     if (rip && rport)
 	w->startRTP(rip,rport.toInteger(),msg);
     msg.setParam("localport",String(w->port()));
     msg.setParam("rtpid",w->id());
 
+    if (msg.getBoolValue("getsession",!msg.userData()))
+	msg.userData(w);
     return true;
 }
 
@@ -785,16 +816,20 @@ YRTPPlugin::~YRTPPlugin()
 
 void YRTPPlugin::statusParams(String& str)
 {
-    String tmp;
     s_mutex.lock();
     str.append("chans=",",") << s_calls.count();
+    s_mutex.unlock();
+}
+
+void YRTPPlugin::statusDetail(String& str)
+{
+    s_mutex.lock();
     ObjList* l = s_calls.skipNull();
     for (; l; l=l->skipNext()) {
 	YRTPWrapper* w = static_cast<YRTPWrapper*>(l->get());
-        tmp.append(w->id(),",") << "=" << w->callId();
+        str.append(w->id(),",") << "=" << w->callId();
     }
     s_mutex.unlock();
-    str << ";" << tmp;
 }
 
 void YRTPPlugin::initialize()
@@ -808,6 +843,11 @@ void YRTPPlugin::initialize()
     s_maxjitter = cfg.getIntValue("general","maxjitter");
     s_tos = cfg.getValue("general","tos");
     s_autoaddr = cfg.getBoolValue("general","autoaddr",true);
+    s_anyssrc = cfg.getBoolValue("general","anyssrc",false);
+    s_rtcp = cfg.getBoolValue("general","rtcp",true);
+    s_drill = cfg.getBoolValue("general","drillhole",Engine::clientMode());
+    s_sleep = cfg.getIntValue("general","defsleep",5);
+    RTPGroup::setMinSleep(cfg.getIntValue("general","minsleep"));
     setup();
     if (m_first) {
 	m_first = false;
