@@ -388,6 +388,11 @@ public:
      */
     virtual ~MessageHandler();
 
+    /**
+     * Destroys the object, performs cleanup first
+     */
+    virtual void destruct();
+
     /**
      * This method is called whenever the registered name matches the message.
      * @param msg The received message
@@ -429,6 +434,7 @@ public:
     void clearFilter();
 
 private:
+    void cleanup();
     unsigned m_priority;
     MessageDispatcher* m_dispatcher;
     NamedString* m_filter;
@@ -687,9 +693,10 @@ public:
      * Running modes - run the engine as Console, Client or Server.
      */
     enum RunMode {
-	Console = 0,
-	Client = 1,
-	Server = 2
+	Stopped = 0,
+	Console = 1,
+	Client = 2,
+	Server = 3,
     };
 
     /**
@@ -723,6 +730,20 @@ public:
      */
     static Engine* self();
 
+    /**
+     * Get the running mode of the engine
+     * @return Engine's run mode as enumerated value
+     */
+    static RunMode mode()
+	{ return s_mode; }
+
+    /**
+     * Check if the engine is running as telephony client
+     * @return True if the engine is running in client mode
+     */
+    inline static bool clientMode()
+	{ return s_mode == Client; }
+
     /**
      * Register or unregister a plugin to the engine.
      * @param plugin A pointer to the plugin to (un)register
@@ -943,6 +964,7 @@ private:
     static String s_extramod;
     static String s_modsuffix;
     static int s_haltcode;
+    static RunMode s_mode;
 };
 
 }; // namespace TelEngine
