@@ -545,13 +545,33 @@ public:
      */
     Thread* thread() const;
 
+    /**
+     * Check if the data thread is running
+     * @return True if the data thread was started and is running
+     */
+    bool running() const;
+
+    /**
+     * Get the current status of the asynchronous deletion flag
+     */
+    inline bool asyncDelete() const
+	{ return m_asyncDelete; }
+
 protected:
     /**
      * Threaded Source constructor
      * @param format Name of the data format, default "slin" (Signed Linear)
      */
     inline ThreadedSource(const char* format = "slin")
-	: DataSource(format), m_thread(0) { }
+	: DataSource(format), m_thread(0), m_asyncDelete(false)
+	{ }
+
+    /**
+     * Derived classes should call this method to let the source to be
+     *  destroyed asynchronously in the data thread
+     */
+    inline void asyncDelete(bool async)
+	{ m_asyncDelete = async; }
 
     /**
      * The worker method. You have to reimplement it as you need
@@ -559,12 +579,20 @@ protected:
     virtual void run() = 0;
 
     /**
-     * The cleanup after thread method
+     * The cleanup after thread method, deletes the source if already
+     *  dereferenced and set for asynchronous deletion
      */
     virtual void cleanup();
 
+    /**
+     * Override so destruction can be delayed after all references were lost
+     *  to let the data pumping thread end normally
+     */
+    virtual void zeroRefs();
+
 private:
     ThreadedSourcePrivate* m_thread;
+    bool m_asyncDelete;
 };
 
 /**
@@ -642,6 +670,26 @@ public:
      */
     static ObjList* destFormats(const DataFormat& sFormat = "slin", int maxCost = -1, unsigned int maxLen = 0, ObjList* lst = 0);
 
+    /**
+     * Get a list of formats supported by transcoding for a given format list
+     * @param formats List of data format names
+     * @param existing Also return formats already existing in the initial list
+     * @param sameRate Only return formats with same sampling rate
+     * @param sameChans Only return formats with same number of channels
+     * @return List of format names, must be freed by the caller
+     */
+    static ObjList* allFormats(const ObjList* formats, bool existing = true, bool sameRate = true, bool sameChans = true);
+
+    /**
+     * Get a list of formats supported by transcoding for a given format list
+     * @param formats Data format names as comma separated list
+     * @param existing Also return formats already existing in the initial list
+     * @param sameRate Only return formats with same sampling rate
+     * @param sameChans Only return formats with same number of channels
+     * @return List of format names, must be freed by the caller
+     */
+    static ObjList* allFormats(const String& formats, bool existing = true, bool sameRate = true, bool sameChans = true);
+
     /**
      * Check if bidirectional conversion can be performed by installed translators
      * @param fmt1 Name of the first data format
@@ -1130,6 +1178,12 @@ protected:
      */
     void setPeer(CallEndpoint* peer, const char* reason = 0, bool notify = true);
 
+    /**
+     * Set a foreign data endpoint in this object
+     * @param endPoint Data endpoint to set, will replace one with same type
+     */
+    void setEndpoint(DataEndpoint* endPoint);
+
     /**
      * Set a new ID for this call endpoint
      * @param newId New ID to set to this call
@@ -1317,9 +1371,18 @@ protected:
     /**
      * Routing message handler that is invoked for all call.route messages.
      * @param msg Call routing message
+     * @return True to stop processing the message, false to try other handlers
      */
     virtual bool msgRoute(Message& msg);
 
+    /**
+     * Handler for special commands and line completion requests.
+     * By default it calls @ref commandExecute() or @ref commandComplete().
+     * @param msg Command message
+     * @return True to stop processing the message, false to try other handlers
+     */
+    virtual bool msgCommand(Message& msg);
+
     /**
      * Build the module identification part of the status answer
      * @param str String variable to fill up
@@ -1332,6 +1395,29 @@ protected:
      */
     virtual void statusParams(String& str);
 
+    /**
+     * Build the details reporting part of the status answer
+     * @param str String variable to fill up
+     */
+    virtual void statusDetail(String& str);
+
+    /**
+     * Execute a specific command
+     * @param retVal String to append the textual command output to
+     * @param line Command line to attempt to execute
+     * @return True to stop processing the message, false to try other handlers
+     */
+    virtual bool commandExecute(String& retVal, const String& line);
+
+    /**
+     * Complete a command line
+     * @param msg Message to return completion into
+     * @param partLine Partial line to complete, excluding the last word
+     * @param partWord Partial word to complete
+     * @return True to stop processing the message, false to try other handlers
+     */
+    virtual bool commandComplete(Message& msg, const String& partLine, const String& partWord);
+
     /**
      * Set the local debugging level
      * @param msg Debug setting message
@@ -1359,6 +1445,7 @@ private:
     bool m_outgoing;
     u_int64_t m_timeout;
     u_int64_t m_maxcall;
+    mutable unsigned int m_sequence;
 
 protected:
     String m_status;
@@ -1396,6 +1483,31 @@ public:
      */
     Message* message(const char* name, bool minimal = false, bool data = false);
 
+    /**
+     * Create a filled notification message, copy some parameters from another message
+     * @param name Name of the message to create
+     * @param original Parameters to copy from, can be NULL
+     * @param params Comma separated list of parameters to copy,
+     *  if NULL will be taken from the "copyparams" parameter of original
+     * @param minimal Set to true to fill in only a minimum of parameters
+     * @param data Set the channel as message data
+     * @return A new allocated and parameter filled message
+     */
+    Message* message(const char* name, const NamedList* original, const char* params = 0, bool minimal = false, bool data = false);
+
+    /**
+     * Create a filled notification message, copy some parameters from another message
+     * @param name Name of the message to create
+     * @param original Parameters to copy from
+     * @param params Comma separated list of parameters to copy,
+     *  if NULL will be taken from the "copyparams" parameter of original
+     * @param minimal Set to true to fill in only a minimum of parameters
+     * @param data Set the channel as message data
+     * @return A new allocated and parameter filled message
+     */
+    inline Message* message(const char* name, const NamedList& original, const char* params = 0, bool minimal = false, bool data = false)
+	{ return message(name,&original,params,minimal,data); }
+
     /**
      * Notification on remote call making some progress, not enabled by default
      * @param msg Notification message
@@ -1455,6 +1567,13 @@ public:
      */
     virtual bool msgUpdate(Message& msg);
 
+    /**
+     * Notification on message masquerade as channel request
+     * @param msg Message already modified to masquerade as this channel
+     * @return True to stop processing the message, false to masquerade it
+     */
+    virtual bool msgMasquerade(Message& msg);
+
     /**
      * Status message handler that is invoked only for messages to this channel
      * @param msg Status message
@@ -1713,10 +1832,10 @@ protected:
     /**
      * Attempt to install a data sniffer to detect inband tones
      * Needs a tone detector module capable of attaching sniffer consumers.
-     * @param sniffer Name of the sniffer to install
+     * @param sniffer Name of the sniffer to install, default will detect all tones
      * @return True on success
      */
-    bool toneDetect(const char* sniffer = "tone/*");
+    bool toneDetect(const char* sniffer = 0);
 
 private:
     void init();
@@ -1889,6 +2008,15 @@ protected:
      */
     virtual bool msgExecute(Message& msg, String& dest) = 0;
 
+    /**
+     * Complete a command line
+     * @param msg Message to return completion into
+     * @param partLine Partial line to complete, excluding the last word
+     * @param partWord Partial word to complete
+     * @return True to stop processing the message, false to try other handlers
+     */
+    virtual bool commandComplete(Message& msg, const String& partLine, const String& partWord);
+
     /**
      * Status message handler that is invoked only for matching messages.
      * @param msg Status message
@@ -1908,10 +2036,10 @@ protected:
     virtual void statusParams(String& str);
 
     /**
-     * Build the clannel list part of the status answer
+     * Build the channel list part of the status answer
      * @param str String variable to fill up
      */
-    virtual void statusChannels(String& str);
+    virtual void statusDetail(String& str);
 
     /**
      * Set the local debugging level
